##########
## Track all of the potential identifier sources of the chemical (InChI is structure, but could be ID)
##########
chemID <- setRefClass("chemID",
  fields = list(
    Compound="vector",    ## All compound names here ##
    CASRN="vector",         ## All known CAS names here##
    GSID="character"      ## Double-ditto ##
  ),
  methods = list(
    getIdentifierColnames = function() {
      return(c("CAS", "CASRN", "GSID", "Compound", "Name", "IUPAC"))
    },
    #####
    ## CASRNs are a semicolon delimited field. This function returns a vector of CAS.
    #####
    vectorize.CAS = function() {
      casvec <- unlist(strsplit(.self$CASRN, ";"))
      return(casvec)
    },
    setIdentifiers = function(df=data.frame) {
      # x, y: standardized methods of representation of ID type -- case insensitive
      x <- toupper(getIdentifierColnames())
      y <- toupper(colnames(df))
      for (i in 1:length(df)) {
        x.idx <- grep(y[i], x)
        setVal <- as.character(df[1, i, with = FALSE])
        setID <- translateID(x[x.idx])
        set_stmt <- paste0('.self$', setID, '<- .self$add', setID, '("', setVal, '")')
        eval(parse(text=set_stmt))
        #cat("chemID set_stmt: ", set_stmt, "\n  setID:", setID, "\n  setVal", setVal, "\n")
      }
    },
    ## helper function to setIdentifiers to give appropriate chemID object
    translateID = function(id) {
      if(id == "CAS" | id == "CASRN") return("CASRN")
      if(id == "COMPOUND" | id == "NAME") return("Compound")
      if(id == "GSID") return(id)
      stop("in translateID: No idea what the chemID value of: ", id, "is. This is a problem.\n")
    },
    SMILES2InChI = function(SMILES) {
      cat("In SMILES2InChI: ", SMILES, "\n")
      #InChI <- blah()
      #return(InChI)
      return("XYZ")
    },
    InChI2SMILES = function (InChI) {
      cat("In InChI2SMILES: ", InChI, "\n")
      #SMILES <- blah()
      #return(SMILES)
      return("ABC")
    },
    setCompound = function(compound) {
      .self$Compound <- unique(c(.self$Compound, compound))
    },
    addCompound = function(compound) {
      .self$Compound <- unique(c(.self$Compound, compound))
    },
    setCASRN = function(casrn) {
      ## Un-be-fing-lievable. Srsly, leading 0's to CAS #'s. Time to re-do the analyses. ##
      casrn <- gsub("^0+", '', casrn)
      casrn <- gsub("\\n", ',', casrn)
      .self$CASRN <- unique(c(.self$CASRN, casrn))
    },
    addCASRN = function(casrn) {
      ## Un-be-fing-lievable. Srsly, leading 0's to CAS #'s. Time to re-do the analyses. ##
      casrn <- gsub("^0+", '', casrn)
      casrn <- gsub("\\n", ',', casrn)
      .self$CASRN <- unique(c(.self$CASRN, casrn))
    },
    setGSID = function(gsid) {
      .self$GSID <- gsid
    },
    addGSID = function(gsid) {
      .self$GSID <- gsid
    },
    #####
    ## CASRN is a string with all CASRNs. This function will return the first one (maybe later allow for input to choose
    ## which CASRN to pick).
    #####
    getCASRN = function() {
      x <- unlist(strsplit(CASRN, ","))[1]
      #cat("********************* getCASRN: ", x, " *****************\n")
      return(x)
    }
  )
)
