#########
## Structural descriptors of the chemicals... Add SDF?
#########
chemStructure <- setRefClass("chemStructure",
  fields = list(
    SMILES="character",                ## Whatever SMILES that comes in.
    S2I2S_SMILES="character",       ## SMILES->InChI->canonical_SMILES
    InChI="character",              ## Non-standard InChI (with atom site ionization active)
    InChIKey="character"
  ),
  methods = list(
    getIdentifierColnames = function() {
      return(c("SMILES", "InChI", "InChIKey", "canonical_SMILES"))
    },
    setIdentifiers = function(df=data.frame) {
      # x, y: standardized methods of representation of ID type -- case insensitive
      x <- toupper(getIdentifierColnames())
      y <- toupper(colnames(df))

      for (i in 1:length(df)) {
        x.idx <- grep(paste0("^", y[i], "$"), x)   ## w/o ^, picks SMILES and canonical_SMILES: w/0 $, InChI and InChIKey
        setVal <- as.character(df[1, i, with = FALSE])  ## data.table. dim(df) = 1,3 if SMILES, InChI, and InChIKey provided.
        # If there is a backslash in the SMILES string, this will be a problem. Following code fixes this issue:
        setVal <- gsub('\\\\', '\\\\\\\\', setVal)
        setID <- translateID(x[x.idx])
        set_stmt <- paste0('.self$', setID, '<- .self$add', setID, '("', setVal, '")')
        eval(parse(text=set_stmt))
        #cat("chemStructure set_stmt: ", set_stmt, "\n  setID:", setID, "\n  setVal", setVal, "\n")
      }
    },
    ## helper function to setIdentifiers to give appropriate chemID object
    translateID = function(id) {
      if (id == "SMILES") return("SMILES")
      if (id == "CANONICAL_SMILES") return("SMILES")
      if (id == "CANONICAL.SMILES") return("SMILES")
      if (id == "CANONICAL") return("SMILES")
      if (id == "INCHI") return("InChI")
      if (id == "INCHIKEY") return("InChIKey")
      stop("in translateID: No idea what the chemID value of: ", id, "is. This is a problem.\n")
    },
    ## Add smiles string to the list of SMILES representation currently known. Uniqueify out if SMILES is a copy.
    setSMILES = function(smiles) {
      .self$SMILES <- smiles
    },
    ## Unique structure
    setInChI = function(inchi) {
      .self$InChI <- inchi
    },
    ## Does this recognize our new format? Probably not, so this is likely built on the canonical InChI
    setInChIKey = function(inchiKey) {
      .self$InChIKey <- inchiKey
    },
    addSMILES = function(smiles) {
      .self$SMILES <- smiles
    },
    ## Unique structure
    addInChI = function(inchi) {
      .self$InChI <- inchi
    },
    ## Does this recognize our new format? Probably not, so this is likely built on the canonical InChI
    addInChIKey = function(inchiKey) {
      .self$InChIKey <- inchiKey
    },
    #####
    ## Utility function. May be useful in the future?
    #####
    getSMILES = function() {
      return(SMILES)
    }
  )
)
