##########
## Script that will make the predictions for SOT PCs.
##########
validate_PBPKxIon <- function() { 
  ## Pass the validation data sheets to Predict_partitioning.
  PCs <- Predict_partitioning(
    chem.data = "/media/nassrvr/Projects/PBPK/Partition_Coefficient/data/validation_data.xlsx",
    chem.data.sheet = "PBPKxIon_Peyret",
    pred.or.input.pKa = "/media/nassrvr/Projects/PBPK/Partition_Coefficient/data/validation_data.xlsx",
    pred.or.input.pKa.sheet = "pKa",
    fingerprint.data = NULL,
    physiology.data = "/media/nassrvr/Projects/PBPK/Partition_Coefficient/data/validation_data.xlsx",
    sheet_physiology.interstitial = "it",
    sheet_physiology.cellular = "ct"
  )
  ReportVerificationData(chem.data = "/media/nassrvr/Projects/PBPK/Partition_Coefficient/data/validation_data.xlsx", PC = PCs)
}

##########
## The following 2 functions are to read in the data sets used for the manuscript:
## 1. 32k exposure universe
## 2. Vdss validation (22 NHANES)
##########
SetExposureUniverse <- function(cc.calcbit = FALSE)
{
  Initialize()
#                chem.data.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/data/xlsdata/32k_pKa_prediction-20150414.xlsx",
#                chem.data.sheet = "chem_data",
#                pKa.data.xls = "//media/nassrvr/Projects/PBPK/pK-2014-09-15/data/xlsdata/32k_pKa_prediction-20150414.xlsx",
#                pKa.data.sheet = "MARVIN",
#                fingerprint.data.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/32k_fingerprints.xlsx",

  data.o <- PBPKxIonSetChemical(
                chem.data.xls = "./32k_pKa_prediction-20150414.xlsx",
                chem.data.sheet = "chem_data",
                pKa.data.xls = "./32k_pKa_prediction-20150414.xlsx",
                pKa.data.sheet = "MARVIN",
                fingerprint.data.xls = "./32k_fingerprints.xlsx",
                fingerprint.data.sheet = 1,
                cc.calcbit = cc.calcbit)

  ## If categories are wanted, need to set the is.far, is.near, etc, and collate with the pKaTools.df.
  if (cc.calcbit) { 
    cats <- data.o$setUseCategories(chem_data = data.o$pKaTools.df) 
    data.o$pKaTools.df <- mergeAndDrop(data.o$pKaTools.df, cats)
  }

  return(data.o)
}

SetNHANESSamples <- function(cc.calcbit = FALSE)
{
  Initialize()
#                chem.data.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx",
#                chem.data.sheet = "chem.data", 
#                pKa.data.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx",
#                pKa.data.sheet = "MARVIN",
#                fingerprint.data = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx",
#                fingerprint.data.sheet = "fingerprints",

  Vdss.o <- PBPKxIonSetChemical(
                chem.data.xls = "./Vdss_predict.xlsx",
                chem.data.sheet = "chem.data", 
                pKa.data.xls = "./Vdss_predict.xlsx",
                pKa.data.sheet = "MARVIN",
                fingerprint.data = "./Vdss_predict.xlsx",
                fingerprint.data.sheet = "fingerprints",
                cc.calcbit = cc.calcbit)

  if (cc.calcbit) {
    cats <- Vdss.o$setUseCategories(chem_data = Vdss.o$pKaTools.df)
    Vdss.o$pKaTools.df <- mergeAndDrop(Vdss.o$pKaTools.df, cats)
  }

  return(Vdss.o)
}

##########
## Data structure creation/population
##########
PBPKxIonSetChemical <- function(
    chem.data.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/data/xlsdata/32k_pKa_prediction-20150414.xlsx",
    chem.data.sheet = "chem_data",
    pKa.data.xls = "//media/nassrvr/Projects/PBPK/pK-2014-09-15/data/xlsdata/32k_pKa_prediction-20150414.xlsx",
    pKa.data.sheet = "MARVIN",
    fingerprint.data.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/32k_fingerprints.xlsx",
    fingerprint.data.sheet = 1,
    cc.calcbit = FALSE
   )
{
  Initialize()
  data.object <- Data$new() 
  ## All chemicals, each chemical unique: chemList
  data.object$SetChemicalData(
    chem.data=chem.data.xls, 
    chem.data.sheet=chem.data.sheet, 
    prediction.data=pKa.data.xls, 
    prediction.sheet=pKa.data.sheet, 
    fingerprint.data=fingerprint.data.xls,
    fingerprint.data.sheet=fingerprint.data.sheet,
    cc.calcbit=cc.calcbit)

  return(data.object)
}

##########
## PhysData structure creation/population
##########
PBPKxIonSetPhysiology <- function(
#    physiology.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx",
    physiology.xls = "./Vdss_predict.xlsx",
    physiology.it.sheet = "it",
    physiology.ct.sheet = "ct",
    physiology.tissue.sheet = "tissue.phys",
    species = "Human"
   )
{
  physiology.object <- PhysData$new()
  ## Create the data object that is to be set with all of our data. ##
  physiology.object$SetTissueData(
    physiology.data = physiology.xls,
    interstitial.sheet = physiology.it.sheet,
    cellular.sheet = physiology.ct.sheet,
    tissue.sheet = physiology.tissue.sheet,
    species = species
  )
 
  return(physiology.object)
}

##########
## Special function for calculating the Vdss for generic input .xls files.
##########
PBPKxIonFullAnalysis <- function(
    chem.data.xls = "/media/nassrvr/Projects/Obach/validation_obach2008.xlsx",
    chem.data.sheet = "chem_data",
    pKa.data.xls = "/media/nassrvr/Projects/Obach/validation_obach2008.xlsx",
    pKa.data.sheet = "MARVIN",
    fingerprint.data.xls = "/media/nassrvr/Projects/Obach/validation_obach2008.xlsx",
    fingerprint.data.sheet = 1,
    physiology.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx",
    physiology.it.sheet = "it",
    physiology.ct.sheet = "ct",
    physiology.tissue.sheet = "tissue.phys",
    validation.data = "/media/nassrvr/Projects/Obach/validation_obach2008.xlsx",
    validation.data.sheet = "pK"
  )
{
  data.object <- PBPKxIonSetChemical(
    chem.data = chem.data.xls, chem.data.sheet = chem.data.sheet,
    pKa.data.xls = pKa.data.xls, pKa.data.sheet = pKa.data.sheet,
    fingerprint.data = fingerprint.data.xls, fingerprint.data.sheet = 1
  )

  physiology.object <- PBPKxIonSetPhysiology(
    physiology.xls = physiology.xls,
    physiology.it.sheet = physiology.it.sheet, physiology.ct.sheet = physiology.ct.sheet,
    physiology.tissue.sheet = physiology.tissue.sheet
  )

  Vss.predict <- Data2Vdss(data.object, physiology.object)
  Vss.validation <- setValidation(validation.data, validation.data.sheet)

  Vss.compare <- cbind(Vss.predict, Vss.validation, by = "CASRN")

  return(Vss.compare)
}

##########
## Calculate the partition coefficients and volume of distribution provided that chemical data and physiology
##########
Data2Vdss <- function(data.o, physiology.o)
{
  ## CALC VDss ##
  dataset.Vdss <- data.frame(CASRN = rep("", length(data.o$chemList)), Vss = rep(0, length(data.o$chemList)), stringsAsFactors = FALSE)
  for (i in seq_along(data.o$chemList)) {
    ## For each "new" chemical, get all PCs for the tissue
    tissue.PC <- list(tissue = names(physiology.o$tissueList))
    for (tissue in seq_along(physiology.o$tissueList)) {
      if (physiology.o$tissueList[[tissue]]$tissueType != "RBC") {
        newPC <- PartitionCoefficient$new()
        newPC$predPC(physiology.o$tissueList[[tissue]], data.o$chemList[[i]])
        tissue.PC[[physiology.o$tissueList[[tissue]]$tissueType]] <- 
                 c(tissue.PC[[physiology.o$tissueList[[tissue]]$tissueType]], newPC)
      }
    }
    ## Then calculate the volume of distribution
    indf <- CalculateVolumeofDistribution(physiology.o, data.o$chemList[[i]], tissue.PC)
    dataset.Vdss$CASRN[i] <- data.o$chemList[[i]]$ID$CASRN
    dataset.Vdss$Vss[i] <- indf$Kp
  }
  ## Holds the volumes of distribution for each chemical.
  return(dataset.Vdss)
}

##########
## Script that runs my compounds through the partitioning.
##########
PBPKxIonPredict <- function(chem.data = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/data/xlsdata/32k_pKa_prediction-20150414.xlsx",
                            chem.data.sheet = "chem_data", 
                            pred.or.input.pKa = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/data/xlsdata/32k_pKa_prediction-20150414.xlsx",
                            pred.or.input.pKa.sheet = "MARVIN",
                            fingerprint.data = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/32k_fingerprints.xlsx"
                           ) 
{
  physiology.object <- PBPKxIonSetPhysiology(
    physiology.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx",
    physiology.it.sheet = "it", 
    physiology.ct.sheet = "ct",
    physiology.tissue.sheet = "tissue.phys"
  )
  data.object <- SetExposureUniverse()

  ## CALC VDss ##
  dataset.Vdss <- data.frame(CASRN = rep("", length(data.object$chemList)), 
                             logP  = rep(0, length(data.object$chemList)), 
                             P.ow  = rep(0, length(data.object$chemList)), 
                             Vss   = rep(0, length(data.object$chemList)), 
                             stringsAsFactors = FALSE)
  for (i in seq_along(data.object$chemList)) {
    ## For each "new" chemical, get all PCs for the tissue
    tissue.PC <- list(tissue = names(physiology.object$tissueList))
    for (tissue in seq_along(physiology.object$tissueList)) {
      if (physiology.object$tissueList[[tissue]]$tissueType != "RBC") {
        newPC <- PartitionCoefficient$new()
        newPC$predPC(physiology.object$tissueList[[tissue]], data.object$chemList[[i]])
        tissue.PC[[physiology.object$tissueList[[tissue]]$tissueType]] <- 
                 c(tissue.PC[[physiology.object$tissueList[[tissue]]$tissueType]], newPC)
      }
    }
    ## Then calculate the volume of distribution
    indf <- CalculateVolumeofDistribution(physiology.object, data.object$chemList[[i]], tissue.PC)
    dataset.Vdss$CASRN[i] <- data.object$chemList[[i]]$ID$CASRN
    dataset.Vdss$logP[i] <- data.object$chemList[[i]]$logP
    dataset.Vdss$P.ow[i] <- data.object$chemList[[i]]$P.ow
    dataset.Vdss$Vss[i] <- indf$Kp
  }
  P.ow.data <<- dataset.Vdss[order(dataset.Vdss$logP),]

  return(dataset.Vdss)
}

RunGenericChemicals <- function(data.object, physiology.object, n = 1000, HLOC.only = FALSE, acid = "", base = "")
{
  run.o <- Data$new()
  generic.chemicals <- GenericChemical.by.logP(data.object, n = n, acid = acid, base = base)
  run.o$SetChemList(generic.chemicals)
  for (j in 1:nrow(generic.chemicals)) {
    ## pKa data.
    run.o$chemList[[j]]$populatePKA(generic.chemicals[j,,drop=FALSE])
  }

  dataset.Vdss <- data.frame(logP = rep(0, length(run.o$chemList)), P.ow = rep(0, length(run.o$chemList)), Vss = rep(0, length(run.o$chemList)), stringsAsFactors = FALSE)
  for (i in seq_along(run.o$chemList)) {
    ## HERE: if HLOC.only is set, then all predictions, regardless of logP, will be HLOC.
    ## If HLOC.only not set, all will be Peyret.
    run.o$chemList[[i]]$is.HLOC = HLOC.only
    run.o$chemList[[i]]$ID$CASRN <- paste0(i,"-xx","-x")
    tissue.PC <- list(tissue = names(physiology.object$tissueList))
    for (tissue in seq_along(physiology.object$tissueList)) {
      if (physiology.object$tissueList[[tissue]]$tissueType != "RBC") {
        newPC <- PartitionCoefficient$new()
        newPC$predPC(physiology.object$tissueList[[tissue]], run.o$chemList[[i]])
        tissue.PC[[physiology.object$tissueList[[tissue]]$tissueType]] <-
                 c(tissue.PC[[physiology.object$tissueList[[tissue]]$tissueType]], newPC)
      }
    }
    indf <- CalculateVolumeofDistribution(physiology.object, run.o$chemList[[i]], tissue.PC)
    dataset.Vdss$logP[i] <- run.o$chemList[[i]]$logP
    dataset.Vdss$P.ow[i] <- run.o$chemList[[i]]$P.ow
    dataset.Vdss$Vss[i] <- indf$Kp
  }

  if (HLOC.only) {
    HLOC.Vdss <<- dataset.Vdss[order(dataset.Vdss$logP),]
    Vdss <- HLOC.Vdss
  } else {
    Peyret.Vdss <<- dataset.Vdss[order(dataset.Vdss$logP),]
    Vdss <- Peyret.Vdss
  }

  return(Vdss)
}

##########
## Calculate the Volume of Distribution for a single chemical
## PC is a data.frame of the partition coefficients for a SINGLE chemical, over all tissues
##########
CalculateVolumeofDistribution <- function(phys.o, chemical, PC) {
  ## For humans ##
  average.BW <- 70   ## Kg
  plasma.volume <- 42.85714    ## ml/kg
  plasma.volume <- plasma.volume / 1000   ## L/Kg

  ## Vdss <- V_p + \sum_{tissue} V_T * Kp_T
  Vdss.df <- data.frame(
    ID = chemical$ID$CASRN, 
    Kp = plasma.volume,
    Ptb = plasma.volume,
    stringsAsFactors = FALSE
  )     ## Plasma volume
  ## Calculate Vdss by multiplying each tissue Kpu by tissue volume.
  ## Tissue volume is in L/kg, plasma volume in mL/kg. Multiply tissueVolume by 1000 
  #cat("CHEMICAL P.ow:", chemical$P.ow, "\n")
  for (j in 1:length(PC$tissue)) {
    if (phys.o$tissueList[[j]]$tissueType != "RBC") {
      #cat("K.p:", PC[[phys.o$tissueList[[j]]$tissueType]][[1]]$K.p, " vol:", phys.o$tissueList[[j]]$tissueVolume, "\n")
      Vdss.df$Kp <- 
        Vdss.df$Kp + PC[[phys.o$tissueList[[j]]$tissueType]][[1]]$K.p * phys.o$tissueList[[j]]$tissueVolume
      Vdss.df$Ptb <-
        Vdss.df$Ptb + PC[[phys.o$tissueList[[j]]$tissueType]][[1]]$P.tb * phys.o$tissueList[[j]]$tissueVolume
    }
  }
  #cat("Total:", Vdss.df$Kp, "\n")
  #cat("\n")

  return(Vdss.df)
}

ReportVerificationData <- function(chem.data, PC) {
  ## Read in the PCs to verify:
  muscle.verify <- read.xls(chem.data, sheet = "Muscle", stringsAsFactors = FALSE)
  adipose.verify <- read.xls(chem.data, sheet = "Adipose", stringsAsFactors = FALSE)
  liver.verify <- read.xls(chem.data, sheet = "Liver", stringsAsFactors = FALSE)
  for (i in 1:nrow(muscle.verify)) {
    ## Chems without CAS are not evaluated.
    if (length(PC[[adipose.verify$CAS[i]]][["Adipose"]][[1]]) != 0) {
      cat("ADIPOSE: ")
      if (adipose.verify$Ptb[i] - PC[[adipose.verify$CAS[i]]][["Adipose"]][[1]]$P.tb < 0.1) {
        cat(adipose.verify$Compound[i], "correct -->  ")
      } else {
        cat(adipose.verify$Compound[i], "INCORRECT -->  ")
      }
      cat(adipose.verify$Ptb[i], " vs. ", PC[[adipose.verify$CAS[i]]][["Adipose"]][[1]]$P.tb, "\n")
      cat("LIVER: ")
      if (liver.verify$Ptb[i] - PC[[liver.verify$CAS[i]]][["Liver"]][[1]]$P.tb < 0.1) {
        cat(liver.verify$Compound[i], "correct -->  ")
      } else {
        cat(liver.verify$Compound[i], "INCORRECT -->  ")
      }
      cat(liver.verify$Ptb[i], " vs. ", PC[[liver.verify$CAS[i]]][["Liver"]][[1]]$P.tb, "\n")
      cat("MUSCLE: ")
      if (muscle.verify$Ptb[i] - PC[[muscle.verify$CAS[i]]][["Muscle"]][[1]]$P.tb < 0.1) {
        cat(muscle.verify$Compound[i], "correct -->  ")
      } else {
        cat(muscle.verify$Compound[i], "INCORRECT -->  ")
      }
      cat(muscle.verify$Ptb[i], " vs. ", PC[[muscle.verify$CAS[i]]][["Muscle"]][[1]]$P.tb, "\n")
    }
  }
}

Data$methods(
  SetChemicalData = function(chem.data, chem.data.sheet, prediction.data, prediction.sheet, 
      fingerprint.data = NULL, fingerprint.data.sheet = 1, cc.calcbit = TRUE
     )
  {
    #####
    ## pKaTools integrated to this point. Work towards PCs.
    ## data.object holds the data heirarchy:
    ## * Data object 
    ##   - chemList: list of chemical objects (chemList[[x]])
    ##     * FUNCTION I.m() -- Ionization for PC
    ##       - pKa object
    ##         . pKa_df = {value, class, SMILES_position, ionizedSMILES}
    ##         . SMILE = Object of type SMILES. Holds SMILES string, relevant methods to ionize, de-ionize, split, etc.
    ##     * chemID = GSID, CASRN, Compound
    ##     * chemStructure = SMILES, InChI/InChIKey
    ##   - logP
    ##   - logPvow -- If provided, else QSAR Pow-->Pvow
    ##   - logD (Schmitt)
    ##   - Henrys_Law -- water:air PC
    ##   - boiling_point -- Some methods use this as a proxy for VOC/SVOC determination
    ##   - charge -- at 7.4
    #####
    ## In the above reads, the chemical data is not included, just pKa. Here, we read the chemical data and file under the chemList.
    .self$ImportChemData(chem.data, sheet = chem.data.sheet)
    .self$Initialize_PBPKxIon(prediction.data, pKa_sheet = prediction.sheet, fingerprint_file = fingerprint.data, fingerprint_sheet = fingerprint.data.sheet, calc_chemical_category=cc.calcbit)
  }
)

PhysData$methods(
  #####
  ## The equivalent function to importchemdata and initialixe_PBPKxIon
  #####
  SetTissueData = function(physiology.data = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx",
                                 interstitial.sheet = "it",
                                 cellular.sheet = "ct",
                                 tissue.sheet = "tissue.phys",
                                 species = "Human") 
  { 
    ## Cut out data that is not species, modify column names to standard.

    tissue.data_ct <- read.xls(physiology.data, sheet = cellular.sheet, stringsAsFactors = FALSE)
    tissue.data_it <- read.xls(physiology.data, sheet = interstitial.sheet, stringsAsFactors = FALSE)
    tissue.data_phys <- read.xls(physiology.data, sheet = tissue.sheet, stringsAsFactors = FALSE)
    physiology.df <- data.frame(
      Tissue = tissue.data_phys$Tissue,
      volume = tissue.data_phys[, grep(paste0(species, "\\.Vol"), names(tissue.data_phys))], 
      flow = tissue.data_phys[, grep(paste0(species, "\\.Flow"), names(tissue.data_phys))],
      species = rep(species, nrow(tissue.data_phys)),
      stringsAsFactors = FALSE
    )
    ## Set "regular" tissues ##
    .self$setTissues(tissue.data_ct, tissue.data_it, physiology.df)
    .self$RestPerfusedTissues(tissue.data_ct, tissue.data_it, physiology.df)
  }
)

#####
## Function that outputs Excel Templates that will be most useful for running pKaTools and PBPKxIon
#####
OutputExcelTemplates <- function(toDirectory = "~", chem_data = TRUE, prediction = TRUE, fingerprint = TRUE, physiology = TRUE) {
  ## Need to incorporate information into the Excel files (and remove the information about read-in):
  ## * vapor.pressure --> vapor.pressure (in Pa)
  ## * Label optional fields as <field> (optional) and required fields <field> (required), and fields that will limit the 
  ##   types of analyses that PBPKxIon can perform.
  ## * 
  if (chem_data) {
    chem <- data.frame(CASRN="", Compound="", SMILES="", MW="", apKa="", apKa.atoms="", bpKa="", bpKa.atoms="", logP="", logD="", 
                       CLint="", fub="", boiling.point="", vapor.pressure="", Henrys.law="", water.air.PC="", blood.air.PC="")
    write.xls(chem, file = paste0(toDirectory, "/chemical_data.xls"))
  }
  if (prediction) {

  }
  if (fingerprint) {

  }
  if (physiology) {

  }
}

#####
## Sets some global parameters. 
#####
Initialize <- function()
{
  options(warn=2)   # 1: warnings as they occur, 2: treat warnings as errors
  setGlobals()
}

setGlobals <- function() {
  ACID <<- 1
  BASE <<- 2
  INFER <<- 3
  ERROR <<- 4
  ## The atoms that are likely to show up in SMILES strings. This is not comprehensive, grow it as needed.
  chemical_atoms <<- c("Si", "cl", "Cl", "Br", "As", "Se", "br", 
                      "c", "C", "n", "N", "o", "O", "p", "P", "f", "F", "s", "S", "I", "B", "b")
  options(warn=2)   # 1: warnings as they occur, 2: treat warnings as errors
  LoadIAResources()
  setSheet()
}

setSheet <- function(pred="MARVIN") {
  ## I hate that I have to do this, but because SPARC is idiotic, I have to know which predictor I will be using in the run.
  oldSPARC <- if (exists("SPARC")) SPARC else ""
  oldMARVIN <- if (exists("MARVIN")) MARVIN else ""
  oldADMETPRED <- if (exists("ADMETPRED")) ADMETPRED else "FALSE"

  ## Now set to new predictor
  SPARC <<- if (grepl("SPARC", pred)) TRUE else FALSE
  MARVIN <<- if (grepl("MARVIN", pred)) TRUE else FALSE
  ADMETPRED <<- if (grepl("ADMET", pred)) TRUE else FALSE
  ## This checks to see if the pred variable in .GlobalEnv is the same as the predictions being asked for.
  same_predictions <- FALSE
  if ( (oldSPARC == TRUE & pred == "SPARC") | (oldMARVIN == TRUE & pred == "MARVIN") | (oldADMETPRED == TRUE | pred == "ADMETPRED") )
    same_predictions <- TRUE

  return(same_predictions)
}
