## This takes the unified data frame created with all of the previous plots and infers ionizable atom type.
## The data frame is: MARVIN_by_pKa_chem_class_fp.flat

#("~/Projects/pK-2014-09-15/pK/projects/pKa-format/data/RData/MARVIN_by_pKa_chem_class_fp_11062014p36.flat.RData")

#####
## Input resources needed for the calculation of Ionizable atom plots.
## May think of making this a class.
#####
LoadIAResources <- function()
{
  ia_dictionary <<- list(
					  AlHdrxl_.OH = c("[^=]\\[O-\\]\\)[ABCDEFGHIJKLMNOPQRSTUVWXYZ]",		## AlHdrxl means hydroxyl sticking out 
                                      "[ABCDEFGHIJKLMNOPQRSTUVWXYZ]\\[O-\\]\\)",			## into solution (away from aliphatic string)
                                      "^\\[O-\\]",			## Terminal conditions
                                      "[^=]\\[O-\\]$"			## Terminal conditions
					  				  #"C\\d*\\[O-\\][^P]",
					  				  #"C\\(*\\[O-\\][^P]", 
					  				  #"\\[O-\\]\\d*C",		
					  				  #"\\[O-\\]\\)*C"
					  				 ),		## Aliphatic C on one side
					  ArHdrxl_.OH = c("c\\d*\\[O-\\]",
					  				  "c\\(*\\[O-\\]", 
					  				  "[^=]\\[O-\\]\\d*c",		## Aromatic c on one side
					  				  "[^=]\\[O-\\]\\)*c"),		
					  Carbonyl_C.O = c("\\(=O\\)\\[C-\\]"),
					  Ether___.O. = c("C\\d*\\[O-\\]\\d*C",
					  				  "C\\d*\\[O-\\]\\)*C",
					  				  "C\\(*\\[O-\\]\\d*C",
					  				  "C\\(*\\[O-\\]\\)*C"
					  				 ),		## C on both sides. Strict subset of AlHdrxl
					  ## One bond with another atom. This searches beginning of string and internal branch ends, i.e., xxxx[NH+])
					  PriAmine_.NH2 = c("^\\[NH3\\+\\]C",
					  					"C\\d*\\[NH3\\+\\]$",
					  					"C\\d*\\[NH\\+\\]\\)", 
					  					"C\\d*\\(\\[NH\\+\\]\\)", 
					  					"\\(\\[NH\\+\\]\\)C",
					  					"C\\)\\[NH3\\+\\]$",
					  					"C\\)\\[NH\\+\\]\\)",
					  					"\\)\\(\\[NH\\+\\]\\)",
					  					"\\[NH\\+\\]\\)c", 
					  					"c\\d*\\[NH\\+\\]\\)",
					  					"^\\[NH3\\+\\]c", 
					  					"c\\d*\\(\\[NH\\+\\]\\)", 
					  					"\\(\\[NH\\+\\]\\)c",
					  					"c\\d*\\[NH3\\+\\]$"
					  				   ),	
					  ## The two following deal with the context surrounding the Primary Amine group.
					  PriAmAli_.NH2 = c("^\\[NH3\\+\\]C",
					  					"C\\d*\\[NH3\\+\\]$",
					  					"C\\d*\\[NH\\+\\]\\)", 
					  					"C\\d*\\(\\[NH\\+\\]\\)", 
					  					"\\(\\[NH\\+\\]\\)C",
					  					"C\\)\\[NH3\\+\\]$",
					  					"C\\)\\[NH\\+\\]\\)",
					  					"\\)\\(\\[NH\\+\\]\\)"
					  				   ),
					  PriAmAro_.NH2 = c("\\[NH\\+\\]\\)c", 
					  					"c\\d*\\[NH\\+\\]\\)",
					  					"^\\[NH3\\+\\]c", 
					  					"c\\d*\\(\\[NH\\+\\]\\)", 
					  					"\\(\\[NH\\+\\]\\)c",
					  					"c\\d*\\[NH3\\+\\]$"
					  				   ),
#					  Amide___C..O.N.lt. = c("\\[N-\\]C\\(=O\\)", 		#72
#					  						 "C\\(=O\\)\\[N-\\]"),		# 48249, 92953-54-3; 
					  SecAmine_.gt.NH = c("\\)\\[NH\\+\\]\\d\\)", #49303
					  					  "\\d\\[NH\\+\\]C", # 48654, 94-92-8
					  					  "\\d\\[NH\\+\\]c", # 48291, 93-46-9
					  					  "\\d\\[NH\\+\\]\\d", # 48786, 942-41-6
					  					  "C\\[NH2\\+\\]C", # 48366, 93064-63-2
					  					  "C\\[NH2\\+\\]c", # 48655, 94-92-8
					  					  "\\(\\[NH\\+\\]C" #49028
#					  					  "\\)\\[NH\\+\\]C", # 48359, 93047-39-3  These two are 3 connections.
#					  					  "\\)\\[NH\\+\\]c", # 48716, 94109-29-2     "    "   "  "    "   "
#					  					  "\\(\\[N-\\]c", #49001               NEGATIVES ARE AMIDEs.
#					  					  "\\)\\[N-\\]C", # 49823
#					  					  "\\(\\[N-\\]C", # Atrazine
#					  					  "\\d\\[N-\\]C", # 48265, 93-13-0
#					  					  "\\)\\[N-\\]P", # 49302
#					  					  "\\d\\[N-\\]c" # 49184
					  					 ),
					  ## >N-, no aromatics. ##
					  TerAmine_.gt.N. = c("C\\[NH\\+\\]\\(", # 48610, 94-24-6, 48057, 92-54-6 -- unionized
#					  					  "\\)\\[NH\\+\\]\\(", # 49843
					  					  "\\)\\[NH\\+\\]C",
					  					  "C\\[NH\\+\\]\\d", # 49834, 48669, 94011-82-2
#					  					  "=\\[NH\\+\\]C", # 48365, 93064-63-2
#					  					  "\\d\\[NH\\+\\]=", # 49836, 99156-66-8
#					  					  "N\\[NH\\+\\]=",
#					  					  "C\\[NH\\+\\]=",
#					  					  "=\\[NH\\+\\]\\d", # 49854, 99248-32-5
					  					  "\\d\\[NH\\+\\]\\(",
					  					  "\\)\\[NH\\+\\]\\d",
#					  					  "=\\[NH\\+\\]c", # 48736, 94134-50-6
					  					  "\\d\\[NH\\+\\]\\d"
					  					 ),		
					  QuaAmine_.gt..N...lt. = "X",					#10
					  ArNitrog_.N. = c("\\[nh\\+\\]"),				#11
					  Imine___.N. = "X",							#12
					  Nitrile_C.N = "X",							#13
					  Isocynd_.N....C.. = "X",						#14
					  Thiol___.SH = "X",							#15
					  ThioCrbl_C.S = "X",							#16
					  Sulfide_.S. = c("X"),			## MARVIN appears to not make sulfide predictions, either. ##
					  Slfonium_.gt..S... = "X",						#18
					  Phsphine_.gt.P. = "X",						#19
					  HydxlAmn_.gt.N.O. = "X",						#20
					  Oxime___.N.O. = "X",							#21
					  Nitroso_.N.O = "X",							#22
					  N.oxide_.gt..N...O.. = "X",					#23
					  Hdrzine_.gt.N.N.lt. = "X",					#24
					  Hdrzone_.N.N.lt = "X", #c("[NH+]N", "N[NH+]", "N([NH+]", "[N-]N", "N[N-]"),
					  Azo_____.N.N. = "X",							## Very few of these exist in the current dataset.
					  					#c("c\\[nh\\+\\]c",			#26
					  					#"\\d\\[nh\\+\\]c",
					  					#"\\d\\[nh\\+\\]\\d",
					  					#"\\)\\[nh\\+\\]c",
					  					#"c\\[nh\\+\\]\\d",
					  					#"\\)\\[nh\\+\\]\\d"),
					  Diazo___..N...N = "X",						#27
					  ThioAmin_.gt.N.S. = "X",						#28
					  Disulfde_.S.S. = "X",							#29
					  Sulfoxde_.gt.S.O = "X",						#30
					  Thioxime_.N.S. = "X",							#31
					  Nitro___.NO2 = "X",							#32
					  Nitrite_.O.N.O = "X",							#33
					  Ntrosam_.gt.N.N.O = "X",						#34
					  Azoxy___.N..O..N. = "X",						#35
					  Azide___.N.N.N = "X",							#36
					  Sulfone_.gt.S..O..O = "X",					#37
					  Sulfinat_S..O.O. = "X",
					  Phsporat_P..O.O. = "X",						#39
					  Triazo__.N.N.N. = "X",
					  Oxadiazo_.N.O.N. = "X",						#41
					  Thiadiaz_.N.S.N. = "X",
					  Triazene_.N.N.N. = "X",						#43
					  Nitrate_.O.NO2 = "X",
					  NNitro__.gt.N.NO2 = "X",						#45
					  Sulfonat_.SO3. = c("S\\(\\[O-\\]\\)\\(=O\\)=O",
					  					 "\\[O-\\]S\\(=O\\)\\(=O\\)"),
					  Sulfite_.SO3.lt. = "X",						#47
					  Sulfonmd_.SO2.N.lt. = "X",
					  PriSlfmd_.SO2.NH2 = "X",						#49
					  Phsponat_.PO3.lt. = "X",
					  SPhspnat_.SPO2.lt. = "X",						#51
					  Phspite_.OPO2.lt. = "X",
					  Tetrazo_.NNNN. = "X",							#53
					  Oxdzoxde_.N..O.ON. = "X",
					  NOHydxam_.N.N.O.OH = "X",						#55
					  Sulfamid_.gt.NSO2N.lt. = "X",
					  Sulfate_.OSO3 = "X",							#57
					  ThioSulf_.SSO3 = "X",
					  Phspate_O.PO3.lt.. = "X",						#59
					  PhspatS_O.PS.k.O.3.k..lt.. = "X",
					  SPhspat_S.PO3.lt.. = "X",						#61
					  SPhspatS_S.PS.k.O.3.k..lt.. = "X",
					  PhspatN_O.PN.n.O.3.n..lt..lt..lt. = "X",		#63
					  SPhspatN_S.PN.n.O.3.n..lt..lt..lt. = "X",
					  PhsptSN_O.PS.k.N.n.O.3.n.k..lt..lt. = "X",	#65
					  SPhsptSN_S.PS.k.N.n.O.3.n.k..lt..lt. = "X",
					  Diphspat_O.PO2.O.PO2.O = "X",					#67
					  Triphspt_O.PO2.O.PO..O..OPO2.O = "X",
					  AlCbxyl_.COOH = c("C\\(\\[O-\\]\\)=O",		#69
					  					"\\[O-\\]C\\(=O\\)"),		
					  ArCbxyl_.COOH = "X",
					  Ester___C..O.OC = c("X"),		#???8 Not ionized?? MARVIN specific, or pKaTools problem?
					  Amide___C..O.N.lt. = c("\\[N-\\]C\\(=O\\)", 		#72
					  						 "C\\(=O\\)\\[N-\\]"),		# 48249, 92953-54-3; 
					  Thioamd_C..S.N.lt. = "X",
					  Amidine_C..N..N.lt. = "X",
					  Isocyant_.N.C.O = "X",
					  Thiocynt_.S.C.N = "X",
					  Isothcnt_.N.C.S = "X",
					  Urea____.gt.NC..O.N.lt. = "X",
					  Carbmte_.gt.NC..O.O.lt. = "X",
					  Guandne_.gt.NC..N..N.lt. = "X",
					  Imide___O.C.N..C.O = "X",
					  Barbitur_C1C..O.NC..O.NC1.O = "X",
					  Hdrzone_.N.N.lt = "X"
					 )

  ia_thesaurus <<- list(
					  AlHdrxl_.OH = "AlHdrxl_-OH",
					  ArHdrxl_.OH = "ArHdrxl_-OH",		
					  Carbonyl_C.O = "Carbonyl_C=O",
					  Ether___.O. = "Ether___-O-",
					  PriAmine_.NH2 = "PriAmine_-NH2",
					  PriAmAli_.NH2 = "PriAmAli_-NH2",
					  PriAmAro_.NH2 = "PriAmAro_-NH2",
					  SecAmine_.gt.NH = "SecAmine_>NH",
					  TerAmine_.gt.N. = "TerAmine_>N-",		
					  QuaAmine_.gt..N...lt. = "X",					#10
					  ArNitrog_.N. = "ArNitrog_=N-",				#11
					  Imine___.N. = "X",							#12
					  Nitrile_C.N = "X",							#13
					  Isocynd_.N....C.. = "X",						#14
					  Thiol___.SH = "X",							#15
					  ThioCrbl_C.S = "X",							#16
					  Sulfide_.S. = c("X"),			## MARVIN appears to not make sulfide predictions, either. ##
					  Slfonium_.gt..S... = "X",						#18
					  Phsphine_.gt.P. = "X",						#19
					  HydxlAmn_.gt.N.O. = "X",						#20
					  Oxime___.N.O. = "X",							#21
					  Nitroso_.N.O = "X",							#22
					  N.oxide_.gt..N...O.. = "X",					#23
					  Hdrzine_.gt.N.N.lt. = "X",					#24
					  Hdrzone_.N.N.lt = "X", #c("[NH+]N", "N[NH+]", "N([NH+]", "[N-]N", "N[N-]"),
					  Azo_____.N.N. = "X",							## Very few of these exist in the current dataset.
					  					#c("c\\[nh\\+\\]c",			#26
					  					#"\\d\\[nh\\+\\]c",
					  					#"\\d\\[nh\\+\\]\\d",
					  					#"\\)\\[nh\\+\\]c",
					  					#"c\\[nh\\+\\]\\d",
					  					#"\\)\\[nh\\+\\]\\d"),
					  Diazo___..N...N = "X",						#27
					  ThioAmin_.gt.N.S. = "X",						#28
					  Disulfde_.S.S. = "X",							#29
					  Sulfoxde_.gt.S.O = "X",						#30
					  Thioxime_.N.S. = "X",							#31
					  Nitro___.NO2 = "X",							#32
					  Nitrite_.O.N.O = "X",							#33
					  Ntrosam_.gt.N.N.O = "X",						#34
					  Azoxy___.N..O..N. = "X",						#35
					  Azide___.N.N.N = "X",							#36
					  Sulfone_.gt.S..O..O = "X",
					  Sulfinat_S..O.O. = "X",
					  Phsporat_P..O.O. = "X",
					  Triazo__.N.N.N. = "X",
					  Oxadiazo_.N.O.N. = "X",
					  Thiadiaz_.N.S.N. = "X",
					  Triazene_.N.N.N. = "X",
					  Nitrate_.O.NO2 = "X",
					  NNitro__.gt.N.NO2 = "X",
					  Sulfonat_.SO3. = c("S\\(\\[O-\\]\\)\\(=O\\)=O",
					  					 "\\[O-\\]S\\(=O\\)\\(=O\\)"),
					  Sulfite_.SO3.lt. = "X",
					  Sulfonmd_.SO2.N.lt. = "X",
					  PriSlfmd_.SO2.NH2 = "X",
					  Phsponat_.PO3.lt. = "X",
					  SPhspnat_.SPO2.lt. = "X",
					  Phspite_.OPO2.lt. = "X",
					  Tetrazo_.NNNN. = "X",
					  Oxdzoxde_.N..O.ON. = "X",
					  NOHydxam_.N.N.O.OH = "X",
					  Sulfamid_.gt.NSO2N.lt. = "X",
					  Sulfate_.OSO3 = "X",
					  ThioSulf_.SSO3 = "X",
					  Phspate_O.PO3.lt.. = "X",
					  PhspatS_O.PS.k.O.3.k..lt.. = "X",
					  SPhspat_S.PO3.lt.. = "X",
					  SPhspatS_S.PS.k.O.3.k..lt.. = "X",
					  PhspatN_O.PN.n.O.3.n..lt..lt..lt. = "X",
					  SPhspatN_S.PN.n.O.3.n..lt..lt..lt. = "X",
					  PhsptSN_O.PS.k.N.n.O.3.n.k..lt..lt. = "X",
					  SPhsptSN_S.PS.k.N.n.O.3.n.k..lt..lt. = "X",
					  Diphspat_O.PO2.O.PO2.O = "X",
					  Triphspt_O.PO2.O.PO..O..OPO2.O = "X",
					  AlCbxyl_.COOH = "AlCbxyl_-COOH",		
					  ArCbxyl_.COOH = "X",
					  Ester___C..O.OC = c("X"),		#???8 Not ionized?? MARVIN specific, or pKaTools problem?
					  Amide___C..O.N.lt. = "Amide___C(=O)N<", 
					  Thioamd_C..S.N.lt. = "X",
					  Amidine_C..N..N.lt. = "X",
					  Isocyant_.N.C.O = "X",
					  Thiocynt_.S.C.N = "X",
					  Isothcnt_.N.C.S = "X",
					  Urea____.gt.NC..O.N.lt. = "X",
					  Carbmte_.gt.NC..O.O.lt. = "X",
					  Guandne_.gt.NC..N..N.lt. = "X",
					  Imide___O.C.N..C.O = "X",
					  Barbitur_C1C..O.NC..O.NC1.O = "X",
					  Hdrzone_.N.N.lt = "X"
					 )
}


###DATA###
## This is a logical place to put the collection of fingerprints for the Data class (for testing pKa effects on Vdss)
###DATA###
Data$methods(
  collectFingerprints = function()
  {
  	## Data has chemList, chemList has pKa, pKa has pKa_df, pKa_df has IAT.type, IAT.type is a character string of pasted
  	## IAT types, named as above list.
  	dist.IAT <- list(IAT.ID = names(ia_dictionary))
  	for (i in 1:length(.self$chemList)) {
  	  ## Chemical has some IATs
  	  if (nrow(.self$chemList[[i]]$pKa$pKa_df) > 0) {
  	  	for (j in 1:nrow(.self$chemList[[i]]$pKa$pKa_df)) {
  	  	  IATs <- unlist(strsplit(.self$chemList[[i]]$pKa$pKa_df$IAT.type[j], ";"))
  	  	  ## IATs are from the currently recognizable IATs above:
  	  	  if (length(IATs) > 0) {
            for (k in 1:length(IATs)) {
              dist.IAT[[IATs[k]]] <- c(dist.IAT[[IATs[k]]], .self$chemList[[i]]$pKa$pKa_df$value[j])
            }
  	  	  }
  	  	}
  	  }
  	}
    return(dist.IAT)
  }
)

#####
## From a given SMILES string, find all possible ionizations and see if they match the ia dictionary.
## I.e., find all possible potential ionizations given the dictionary above.
#####
putativeIAT <- function(inSMILES) {
	## A data frame built to replace the pKa_df for chemicals. ##
	putative.pKa.df <- data.frame(value=NULL, class=NULL, SMILES.position=NULL, ionizedSMILES= NULL, IAT.type=NULL, stringsAsFactors=FALSE)
	smiles.o <- SMILES$new(SMILES=inSMILES)
	setGlobals()
	num.atoms <- smiles.o$atomCount()
    for (i in 1:num.atoms) {
    	## Only positions containing an atom have any chance of being ionized ##
    	if (smiles.o$hydrogenCount(i) > 0) {
    	  acid.ion <- smiles.o$ionize(i, ACID)
    	  base.ion <- smiles.o$ionize(i, BASE)
    	  ## Have ionized strings; now check the dictionary to see if any of the patterns match.
    	  acid <- testIonization(acid.ion, paste.it=TRUE)
    	  base <- testIonization(base.ion, paste.it=TRUE)
    	  if (acid != "") {
    	    newRow <- data.frame(value=NA, class=ACID, SMILES.position=i, ionizedSMILES=acid.ion, IAT.type=acid, stringsAsFactors=FALSE)
            putative.pKa.df <- rbind(putative.pKa.df, newRow)
    	  }
    	  if (base != "") {
    	    newRow <- data.frame(value=NA, class=BASE, SMILES.position=i, ionizedSMILES=base.ion, IAT.type=base, stringsAsFactors=FALSE)
            putative.pKa.df <- rbind(putative.pKa.df, newRow)
          }
    	}
    }
    return(putative.pKa.df)
}

#####
## Apply all ionization regular expressions from IA dictionary on an input SMILES string.
#####
testIonization <- function(inSMILES, paste.it = FALSE) {  
  ionization.types <- vector()
  for(i in 1:length(ia_dictionary)) {
    ## cycle through all of the regular expressions.
    exists <- FALSE
    for (j in 1:length(ia_dictionary[[i]])) {
      if (grepl(ia_dictionary[[i]][j], inSMILES)) {
        ionization.types <- c(ionization.types, names(ia_dictionary)[i])
      }
    }
  }
  
  if (length(ionization.types) > 0) {
    cat (inSMILES, "    ", ionization.types, "\n")
  }
  if (paste.it & length(ionization.types) > 0) {
  	if (names(ia_dictionary)[69] %in% ionization.types) {
  	  ## Remove hydroxyl positions from those that are carboxyls.
      if (names(ia_dictionary)[1] %in% ionization.types) {
        hdrxl_pos <- which(ionization.types == names(ia_dictionary)[1])
        ionization.types <- ionization.types[-hdrxl_pos]
      } 
      ## Remove ether positions from those that are carboxyls.
      if (names(ia_dictionary)[4] %in% ionization.types) {
        ether_pos <- which(ionization.types == names(ia_dictionary)[4])
        ionization.types <- ionization.types[-ether_pos]
  	  }
	}
	ionization.types <- unique(ionization.types)
  	ionization.types <- paste(ionization.types, collapse = ";")
  } else {
  	ionization.types <- ""
  }

  return(ionization.types)
}

#####
## Print the ionizable atom types.
#####
listIAT <- function() 
{
	df <- data.frame(num=1:length(ia_dictionary), IAT=names(ia_dictionary))
	print(df)
}

#####
## Print the SMILES strings for each instance of an ionizable atom type. Can make a dictionary of the appropriate 
## ionizations.
#####
printAllIAT <- function(df, which_ionizable_atom) {
  	first_atom_type <- which(names(df) == "AlHdrxl_.OH")
  	last_atom_type <- which(names(df) == "Barbitur_C1C..O.NC..O.NC1.O")
	ionizable_atom.df <- df[,first_atom_type:last_atom_type]
	instances <- which(ionizable_atom.df[,which_ionizable_atom] > 0)
	for (i in 1:length(instances)) {
		extant_ionizations <- names(ionizable_atom.df[instances[i], which(ionizable_atom.df[instances[i],] > 0)])
		cat("Compound: ", df$CASRN[instances[i]], "line: ", instances[i], "  ", i, "/", length(instances),
			"\npKa: ", df$pKa[instances[i]],
			"\nSMILES:   ", df$SMILES[instances[i]],
			"\nSMILES_position:   ", df$SMILES_position[instances[i]],
			"\nionizable_atom.df: ", extant_ionizations,
			"\n\n"
		   )
	}
}

#####
## Given a string IAT and a range of pH, output all SMILES that match.
#####
printIAT <- function(in_flatfile, IAT, pH_range_begin = -10, pH_range_end = 20) {
	## Grab all lines from the flatfile that have the IAT associated with it.
	ia_col <- which(names(in_flatfile) == IAT)
	if (length(ia_col) == 0) { stop("Invalid IAT: \"", IAT, "\"\n")}
	all.iat <- in_flatfile[which(in_flatfile[,which(names(in_flatfile) == IAT)] > 0), ]

	iat.df <- data.frame(pKa=NULL, SMILES=NULL, SMILES_position=NULL, class=NULL, stringsAsFactors=FALSE)

	## Print out only those that are ionized.
	for (i in 1:nrow(all.iat)) {
		if (is.ionized(IAT, all.iat$SMILES[i])) {
			a.df <- data.frame(pKa=all.iat$pKa[i], SMILES=all.iat$SMILES[i],
							   SMILES_position=all.iat$SMILES_position[i], class=all.iat$class[i], stringsAsFactors=FALSE)
			## Keep only those that are within our range.
			if (a.df$pKa < pH_range_end & a.df$pKa > pH_range_begin) {
				if (!is.functional_subgroup(IAT, all.iat$SMILES[i]) ) 
					iat.df <- rbind(iat.df, a.df)
			}
		}
	}
	return(iat.df)
}

#####
## This function checks to see if a particular IAT is a functional subgroup of another one (by SMILES regex). 
## The example that this is AlHdrxl and carboxyls.
#####
is.functional_subgroup <- function(IAT, querySMILES) {
	if (IAT == "AlHdrxl_.OH") {
		if (is.ionized("AlCbxyl_.COOH", querySMILES)) return(TRUE)
	}
	return(FALSE)
}

#####
## Uses the ia_dictionary to determine whether a particular SMILES string is ionized by a specific IAT or not.
#####
is.ionized <- function(IAT, querySMILES) {
	col <- which(names(ia_dictionary) == IAT) 
	if (length(col) != 1) { stop("What is happening in is.ionized??\n")}
	for (j in 1:length(ia_dictionary[[col]])) {
		## The SMILES string has a regex in ia_dictionary that matches for the specific IAT.
		if (grepl(ia_dictionary[[col]][j], querySMILES)) return(TRUE)
	}
	## If none of the ia_dictionary terms catches the IAT, then return false.
	return(FALSE)
}

##########
## Count the ionizable atom types from the flat file.
## is.in.other.IAT.flag is a flag that signals to return the number of total_carboxyls there are that are classified
## hydroxyls (at least, for now).
##########
count.ionizable.atom.types <- function(df, which_ionizable_atom, is.in.other.IAT.flag = FALSE) {
  	first_atom_type <- which(names(df) == "AlHdrxl_.OH")
  	last_atom_type <- which(names(df) == "Barbitur_C1C..O.NC..O.NC1.O")
	ionizable_atom.df <- df[,first_atom_type:last_atom_type]
	instances <- which(ionizable_atom.df[,which_ionizable_atom] > 0)

	total_carboxyl <- 0

	## Structure to collect all of the pKas for each ionizable atom type. ##
	tpcc <- data.frame(
					   pKa=rep(NA,length(instances)), 
					   chem_class=rep(NA,length(instances)),
					   stringsAsFactors=FALSE
					  )
	if (length(instances) > 0) {
		for (i in 1:length(instances)) {
			extant_ionizations <- names(ionizable_atom.df[instances[i], which(ionizable_atom.df[instances[i],] > 0)])
			## cycle through all of the regular expressions.
			exists <- FALSE
			for (j in 1:length(ia_dictionary[[which_ionizable_atom]])) {
				cbxl2 <- FALSE
				if (grepl(ia_dictionary[[which_ionizable_atom]][j], df$SMILES[instances[i]])) {
					if (which_ionizable_atom == 1) {
						## If AlHdrxl, need to check to see if it is also a carboxyl. If so, kill it. ##
						for (k in seq_along(ia_dictionary[[69]])) {
							if (grepl(ia_dictionary[[69]][k], df$SMILES[instances[i]])) {
								cbxl2 <- TRUE
                            	total_carboxyl <- total_carboxyl + 1
								break
							}
							## Not found to be a carboxyl, so it is hydroxyl ##
						}
					}
					if (which_ionizable_atom == 4) {
						## If AlHdrxl, need to check to see if it is also a carboxyl. If so, kill it. ##
						for (k in seq_along(ia_dictionary[[69]])) {
							if (grepl(ia_dictionary[[69]][k], df$SMILES[instances[i]])) {
								cbxl2 <- TRUE
                            	total_carboxyl <- total_carboxyl + 1
								break
							}
							## Not found to be a carboxyl, so it is hydroxyl or ether ##
						}
					}
					if (!cbxl2) {
						exists <- TRUE
						break
					}
				}
			}

			if (exists) {
				## If we are not testing chem classes, then we count pKa as "unspecified"
				if (!("is.Rx" %in% names(df))) {
					tpcc$pKa[i] <- df$pKa[instances[i]]
					tpcc$chem_class[i] <- "unspecified"
				} else {
					if (df$is.Rx[instances[i]]) {
						tpcc$pKa[i] <- df$pKa[instances[i]]
						tpcc$chem_class[i] <- "Rx"
					} else if (df$is.near[instances[i]]) {
						tpcc$pKa[i] <- df$pKa[instances[i]]
						tpcc$chem_class[i] <- "near"
					} else if (df$is.far[instances[i]]) { 
						tpcc$pKa[i] <- df$pKa[instances[i]]
						tpcc$chem_class[i] <- "far"
					}
				}
			}
		}
	}  ## length(instances) == 0 ##

	if (is.in.other.IAT.flag) {
		return(total_carboxyl)
	}

	tpcc <- tpcc[which(!is.na(tpcc$pKa)),]
	return(tpcc)
}

#####
## INPUT:
## * df = data object after running flatten.pKa()
## * which_ionizable_atom = One occurrence of the IAT from ia.dictionary (through needed hists)
##
## This function is called for each of the IATs of interest for each pKa type to determine of there
#####
count.putative.ionizable <- function(df, which_ionizable_atom) {
  	first_atom_type <- which(names(df) == "AlHdrxl_.OH")
  	last_atom_type <- which(names(df) == "Barbitur_C1C..O.NC..O.NC1.O")
	ionizable_atom.df <- df[,first_atom_type:last_atom_type]
	putative_ions <- 0
	current_CASRN <- ""
	for(i in 1:nrow(ionizable_atom.df)) {
		if (df$CASRN[i] != current_CASRN) {
			current_CASRN <- df$CASRN[i]
			if (!is.na(ionizable_atom.df[i, which_ionizable_atom]))
				putative_ions <- putative_ions + ionizable_atom.df[i, which_ionizable_atom]
		}
	}
	return(putative_ions)
}

#####
## Print specific plots
#####
#	hist_data <- count_ionizable_atom_types(1)
#	a <- ggplot(data=hist_data, aes(x=pKa, fill=chem_class)) 
#	a <- a + geom_bar(binwidth=0.25) 
#	a <- a + scale_x_continuous(limits=c(0,15), name="pH") 
#	a <- a + scale_y_continuous(name="number of pKa equal to pH") 
#	a <- a + ggtitle(names(ionizable_atom.df[1])) 
#	a <- a + theme(plot.title = element_text(face="bold", size=32), axis.text.x = element_text(size=20), axis.text.y=element_text(size=20), axis.title.y=element_text(size=24), axis.title.x=element_text(size=24))
#	ggsave(filename=paste0("barplot_X2.png"), plot=a)


#####
## Lists the total number of IATs (n) and the number of times occurrence is ionized:
##  AlHdrxl_.OH   4267 / 9971 = 0.427941
##  ... 
## INPUT:
## * df  data object after running flatten.pKa()
## * needed_hists  Integer vector containing the numbers of the pKa types to calculate, as given in the list ia.dictionary
#####
#needed_hists <- c(8)
calc.percent.ionized <- function(df, needed_hists) {
	ret.df <- data.frame(IAT.type = NULL, percent_ionized = NULL)
  	first_atom_type <- which(names(df) == "AlHdrxl_.OH")
  	last_atom_type <- which(names(df) == "Barbitur_C1C..O.NC..O.NC1.O")
	ionizable_atom.df <- df[,first_atom_type:last_atom_type]
	for (i in 1:length(needed_hists)) {
		x <- count.ionizable.atom.types(df, needed_hists[i])
		total_ionizable <- count.putative.ionizable(df, needed_hists[i])
		## Remove the IATs from a different IAT class from the total.
		total_ionizable <- total_ionizable - count.ionizable.atom.types(df, needed_hists[i], is.in.other.IAT.flag = TRUE)
		percent_ionized <- nrow(x) / total_ionizable
		cat(names(ionizable_atom.df[needed_hists[i]]), " ", nrow(x), "/", total_ionizable, "=", percent_ionized, "\n")
		x.df <- data.frame(IAT.type = names(df[needed_hists[i]]), percent_ionized = percent_ionized)
		ret.df <- rbind(ret.df, x.df)
	}
	return (ret.df)
}

#####
## Get pKa dist and percent of time ionized
## flat.df is the data.frame returned after running flatten.pKa(), i.e., data.o$flatten.pKa())
#####
GetMonteCarloIATInfo <- function(flat.df, needed_hists)
{
  LoadIAResources()
  IAT.list <- list(IAT.type = vector(), percent.ionized = vector(), pKa.list = list())
  for (i in seq_along(needed_hists)) {
  	IAT.list$percent.ionized[i] <- calc.percent.ionized(flat.df, needed_hists[i])$percent_ionized
  	IAT.list$IAT.type[i] <- names(ia_dictionary)[needed_hists[i]]
  	IAT.list$pKa.list[i] <- list(pKa=CollectIATDistributions(flat.df, needed_hists[i])$pKa.dist)
  }
  return(IAT.list)
}

#####
## This does a complete profile of the ionization status of the list of chemicals in the input df, assuming pKaTools format.
## Lists:
##  * #zwitterions
##    ? all weak acid/base ?		## Inferrable????
##    ? Strong acid/base present?   ## Need much more advanced methods.
##  * #neutral
##  * #acid
##  * #base
#####
profile.ionization <- function(in.df) {
	zwitterion <- zwit_weak <- neutral <- acid <- base <- 0

	for (i in 1:nrow(in.df)) {
		type <- ionizableChemType(apKa = in.df$pKaTools_APKA, bpKa = in.df$pKaTools_BPKA)
		if (type == "neutral") neutral <- neutral+1
		else if (type == "acid") acid <- acid+1
		else if (type == "base") base <- base+1
		else if (type == "zwit_weak") zwit_weak <- zwit_weak+1
		else zwitterion <- zwitterion+1
	}

	cat("Ionization Profile:\n")
	cat("  zwitterion: ", zwitterion, "\n")
	cat("    zwit_weak:", zwit_weak, "\n")
	cat("  neutral:    ", neutral, "\n")
	cat("  acid:       ", acid, "\n")
	cat("  base:       ", base, "\n")
}

#####
## Calculate chemical type by entry of apKa and bpKa in pKaTools format.
#####
ionizableChemType <- function(apKa = NULL, bpKa = NULL) {
    nacid <- 0
    nbase <- 0
	if (!is.null(apKa)) {
	  acidVec <- unlist(strsplit(apKa, ";"))
      nacid <- length(acidVec)
	}
	if (!is.null(bpKa)) {
	  baseVec <- unlist(strsplit(bpKa, ";"))
      nbase <- length(baseVec)
	}
	strong_acid <- FALSE
	strong_base <- FALSE
	if (nacid == 0 & nbase == 0) return("neutral")
	else if (nacid > 0 & nbase == 0) return("acid")
	else if (nacid == 0 & nbase > 0) return("base")
	else if (nacid > 0 & nbase > 0) {
		for (j in 1:nacid) if (acidVec[j] < 7) strong_acid <- TRUE
		for (j in 1:nbase) if (baseVec[j] > 7) strong_base <- TRUE
		if (!strong_acid & !strong_base) return("zwit_weak")
		else return("zwitterion")
	}
}

#####
## This function calculates the neutral points of a particular chemical over the range of pH values 0:14, at a certain granularity
## This sum will represent the fraction of a chemical that is neutral at a particular pH, helping to profile the chemical class
## pKas in a different way.
## In the current incarnation, acids, bases, neutrals, and weak zwitterions can be profiled. Potentially, strong zwitterions may be
## possible to add, but strong+weak zwitterions need more complex formulae to actually work.
#####
calculate.neutral.point <- function(in.df, num_points_eval = 140, type = "near") {
	tmpKa <- pKa$new()		## Using frac_species in the pKa class to do calculations.
	step.size <- 14/num_points_eval
	df <- data.frame(pH = seq(step.size,14,step.size), frac.neutral = rep(0, num_points_eval))
	num_chemical <- nrow(in.df)
	for (i in 1:nrow(in.df)) {
		cat("\n", i)
		which.species <- getFracSpecNumerator(in.df$pKaTools_APKA[i], in.df$pKaTools_BPKA[i])
		## Need to get chemical type in order to calculate which.species
		if (which.species == -1) df$frac.neutral <- df$frac.neutral + 1
		else if (which.species == -2) {
			num_chemical <- num_chemical - 1 		## This is a zwitterion, and is not considered, so do not count it as a contributor.
		} else {
			a <- unlist(strsplit(in.df$pKaTools_APKA[i], ";"))
			b <- unlist(strsplit(in.df$pKaTools_BPKA[i], ";"))
			all <- c(a,b)
			i__pKa__ <- as.numeric(sort(all))
			df$frac.neutral <- df$frac.neutral + tmpKa$frac_species(seq(step.size, 14, step.size), which.species, i__pKa__)
		}
		#print(df)
	}
	df$frac.neutral <- df$frac.neutral / num_chemical

	if (type == "near") near.data <<-df
	else if (type == "far") far.data <<-df
	else Rx.data <<-df

	outfile <- paste0(type, ".png")
	png(filename=outfile, bg="white")
	plot(df, type="l", main=paste0(type, " Fraction Neutral by pH"), ylab="Fraction of Uncharged Chemical",ylim=c(0,1))
	dev.off()

	outfile <- paste0(type, ".tiff")
	tiff(filename=outfile, bg="white")
	plot(df, type="l", main=paste0(type, " Fraction Neutral by pH"), ylab="Fraction of Uncharged Chemical", ylim=c(0,1))
	dev.off()

}

#####
## Helper function of calculate.neutral.point. This function applies the rules for getting the numerator of the frac_species function
## That will successfully reproduce the distribution of chemical at neutral for any pH.
#####
getFracSpecNumerator <- function(apKa, bpKa) {
	type <- ionizableChemType(apKa, bpKa)
	if (type == "neutral") return(-1)
	if (type == "acid") return(0)
	if (type == "base" | type == "zwit_weak") return(length(unlist(strsplit(bpKa, ";"))))
	else return(-2)		## zwitterion, can't do it... yet.
}