PhysData <- setRefClass("PhysData",
  fields = list(
    tissueList = "list"
  ),
  methods = list(
    #####
    ## Function to cycle over all tissues on the list and set the data points.
    #####
    setTissues = function(ct.df, it.df, tissue.phys) {
  	  if (nrow(ct.df) != nrow(it.df)) {
  	  	stop("Cellular data must have same number of tissues as interstitial fluid data.\n")
  	  }
  	  inTissue <- ""
  	  ## Tissue PCs require the erythrocyte and plasma data, which are subsets of the entire ct and it data frames.
  	  ## Beyond this function, passing only a specific tissue, so we need to excise these data and also pass them.
  	  ## I.e., calculating Liver: pass ct.df[["Liver"]], it.df[["Liver"]], ct.df[["Erythrocyte"]], it.df[["Plasma"]],
  	  ## but none of the other tissues will be available to setTissueData (no Adipose, Muscle, etc.)
  	  e.df <- ct.df[which(ct.df$Tissue == "Erythrocyte"),]
  	  p.df <- it.df[which(it.df$Tissue == "Plasma"),]
  	  for (i in 1:nrow(ct.df)) {
  	  	if (ct.df$Tissue[i] == "Erythrocyte") inTissue <- "RBC"
  	  	else inTissue <- ct.df$Tissue[i]
  	  	tisphys.row <- which(tissue.phys$Tissue == ct.df$Tissue[i])
  	  	it.row <- which(it.df$Tissue == ct.df$Tissue[i])
  	  	.self$populateTissueData(inTissue, tissue.phys[tisphys.row,], ct.df[i,], it.df[it.row,], e.df, p.df)
  	  }
    },
    populateTissueData = function(tissue.type, tissue.phys.df, ct.df, it.df, e.df, p.df) {
      newTissue <- Tissue$new()
      newTissue$setTissue(tissue.type, tissue.phys.df, ct.df, it.df, e.df, p.df)
      set_stmt <- paste0(".self$tissueList <- c(.self$tissueList, ", tissue.type, " = newTissue)")
      cat(set_stmt, "\n")
      eval(parse(text = set_stmt))
    },
   	testTissue = function() {
  	  ct.df <- read.xls("/media//nassrvr//Projects//PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx", stringsAsFactors=FALSE, sheet="ct")
  	  it.df <- read.xls("/media//nassrvr//Projects//PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx", stringsAsFactors=FALSE, sheet="it")
  	  if (nrow(ct.df) != nrow(it.df)) {
  	  	stop("Cellular data must have same number of tissues as interstitial fluid data.\n")
  	  }
  	  inTissue <- ""
  	  e.df <- ct.df[which(ct.df$Tissue == "Erythrocyte"),]
  	  p.df <- it.df[which(it.df$Tissue == "Plasma"),]
  	  for (i in 1:nrow(ct.df)) {
  	  	if (ct.df$Tissue[i] == "Erythrocyte") inTissue <- "RBC"
  	  	else inTissue <- ct.df$Tissue[i]
  	  	.self$setTissueData(inTissue, ct.df[i,], it.df[i,])
  	  }
  	},
  	##########
  	## Function to set the make-up of the rest of the perfusable tissues:
  	##########
  	RestPerfusedTissues = function(ct.df, it.df, tissue.phys) {
      restTissue <- Tissue$new()
      #for (i in 1:length(.self$tissueList)) {
      #	tissues <- c(tissues, .self$tissueList[[i]]$tissueType)
      #}
      ct.matrices <- c("Fm", "Fnlm", "Fnplm", "Faplm", "Fwm", "Fprm", "pH")
      ct <- ct.df[, ct.matrices]
      it.matrices <- c("Fm", "Fnlm", "Fnplm", "Faplm", "Fwm", "Flprp", "Falbp", "pH")
      it <- it.df[, it.matrices]
      ## Average out columns (multiplied by their tissue volumes), and take the mean volume.
  	  e.df <- ct.df[which(ct.df$Tissue == "Erythrocyte"),]
  	  p.df <- it.df[which(it.df$Tissue == "Plasma"),]
      rest.ct <- apply(ct, MARGIN=2, mean)		## MARGIN specifies column means (1=rows, 2=columns)
      rest.it <- apply(it, MARGIN=2, mean)
      ## Make into data frame for tissue call ##
      rest.ct.df <- data.frame(Fm=rest.ct[which(names(rest.ct) == "Fm")], 
      						   Fnlm=rest.ct[which(names(rest.ct) == "Fnlm")], 
      						   Fnplm=rest.ct[which(names(rest.ct) == "Fnplm")], 
      						   Faplm=rest.ct[which(names(rest.ct) == "Faplm")], 
      						   Fwm=rest.ct[which(names(rest.ct) == "Fwm")], 
      						   Fprm=rest.ct[which(names(rest.ct) == "Fprm")], 
      						   pH=rest.ct[which(names(rest.ct) == "pH")], 
      						   stringsAsFactors=FALSE
      						  )
      rest.it.df <- data.frame(Fm=rest.it[which(names(rest.it) == "Fm")], 
      						   Fnlm=rest.it[which(names(rest.it) == "Fnlm")], 
      						   Fnplm=rest.it[which(names(rest.it) == "Fnplm")], 
      						   Faplm=rest.it[which(names(rest.it) == "Faplm")], 
      						   Fwm=rest.it[which(names(rest.it) == "Fwm")], 
      						   Flprp=rest.it[which(names(rest.it) == "Flprp")], 
      						   Falbp=rest.it[which(names(rest.it) == "Falbp")], 
      						   pH=rest.it[which(names(rest.it) == "pH")], 
      						   stringsAsFactors=FALSE
      						  )
      restTissue$setTissue("Rest", tissue.phys[which(tissue.phys$Tissue == "Rest"),], rest.ct.df, rest.it.df, e.df, p.df)
      .self$tissueList <- c(.self$tissueList, Rest = restTissue)
  	},
  	##########
  	## Sometimes it is nice to have a vector of tissues that are under consideration
  	##########
    returnTissues = function() {
   	  tiss <- vector()
      for (i in seq_along(.self$tissueList)) {
        tiss <- c(tiss, .self$tissueList[[i]])
      }

      return(tiss)
    }
  )
)

