### Sensitivity analysis.

BuildSensitivity <- function(reps = 1002, compounds = NULL)
{
  phys.o <<- PBPKxIonSetPhysiology(
    physiology.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx",
    physiology.it.sheet = "it",
    physiology.ct.sheet = "ct",
    physiology.tissue.sheet = "tissue.phys"
  )
  data.o <<- PBPKxIonSetChemical(
    chem.data.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx",
    chem.data.sheet = "chem.data",
    pKa.data.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx",
    pKa.data.sheet = "MARVIN",
    fingerprint.data.xls = "/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/Vdss_predict.xlsx"
  )

  ## Load the IAT distribution.
  #load("/media/nassrvr/Projects/PBPK/pK-2014-09-15/pK/projects/pKa-format/data/RData/IAT.pKa.distribution.RData")

  ## Initialize a matrix that will hold all chemical by IAT Vdss combinations.
  ### To make new PC entry into this list:
  ### PC[["80-05-7"]][["AlCbxyl_.COOH"]] <- NULL
  ### PC[["80-05-7"]][["AlCbxyl_.COOH"]] <- c(PC[["80-05-7"]][["AlCbxyl_.COOH"]], returned.Vdss)
  IAT.storage <- list(chemicals = data.o$pKaTools.df$CASRN, IAT.type = names(IAT.dist))
  neutral <- list(chemicals = data.o$pKaTools.df$CASRN)		## The neutral Vdss of a compound.
  unchanged <- list(chemicals = data.o$pKaTools.df$CASRN)		## The neutral Vdss of a compound.

  IAT.df <- data.frame(Name = NULL, mod = NULL, stringsAsFactors=FALSE)
  neutral.df <- data.frame(Name = NULL, neutral = NULL, stringsAsFactors=FALSE)
  canonical.df <- data.frame(Name = NULL, canonical = NULL, stringsAsFactors=FALSE)

  if (is.null(compounds)) {
  	## If nothing entered, assume all tissues.
  	compounds <- data.o$pKaTools.df$Compound
  }

  ## Now, calculate the IAT Vdss when changed ##
  for (true.chem in seq_along(data.o$chemList)) {
  	if ( !(data.o$chemList[[true.chem]]$ID$Compound %in% compounds) )
  	  next
  	num.pKa <- nrow(data.o$chemList[[true.chem]]$pKa$pKa_df)
    if (num.pKa > 0) {
      for (which.pKa.it in 1:num.pKa) {
      	pKako.IATs <- unlist(strsplit(data.o$chemList[[true.chem]]$pKa$pKa_df$IAT.type[which.pKa.it], ";"))
      	for (which.IAT.it in seq_along(pKako.IATs)) {
      	  cat("Chemical:", data.o$chemList[[true.chem]]$ID$Compound, "  pKa #:", which.pKa.it, "   of type:", pKako.IATs[which.IAT.it], "\n")
      	  print(data.o$chemList[[true.chem]]$pKa$pKa_df)
      	  ## Send the chemical, the distribution to draw replacement pKa's from, and the number of draws to perform
          sens.data.object <- pKaSensitivity(data.o$chemList[[true.chem]], which.pKa.it, IAT.dist[[pKako.IATs[which.IAT.it]]], reps)
          Vdss <- SensitivityAnalysis(phys.o, sens.data.object)
          IAT.storage[[data.o$chemList[[true.chem]]$ID$CASRN]][[pKako.IATs[which.IAT.it]]] <- 
            c(IAT.storage[[data.o$chemList[[true.chem]]$ID$CASRN]][[pKako.IATs[which.IAT.it]]], Vdss$mod.chem)
          ## Insert the relevant information:
          label <- paste(data.o$chemList[[true.chem]]$ID$Compound, pKako.IATs[which.IAT.it], data.o$chemList[[true.chem]]$pKa$pKa_df$value[which.pKa.it], sep = "  ")
          reps.df <- data.frame(Name = rep(label, length(Vdss$mod.chem)), mod = Vdss$mod.chem, stringsAsFactors=FALSE)
          IAT.df <- rbind(IAT.df, reps.df)
          neut.df <- data.frame(Name = label, neutral = Vdss$neutral, stringsAsFactors=FALSE)
          neutral.df <- rbind(neutral.df, neut.df)
          canon.df <- data.frame(Name = label, canonical = Vdss$canonical, stringsAsFactors=FALSE)
          canonical.df <- rbind(canonical.df, canon.df)
        }
        neutral[[data.o$chemList[[true.chem]]$ID$CASRN]] <- c(neutral[[data.o$chemList[[true.chem]]$ID$CASRN]], Vdss$neutral)
        unchanged[[data.o$chemList[[true.chem]]$ID$CASRN]] <- c(unchanged[[data.o$chemList[[true.chem]]$ID$CASRN]], Vdss$canonical)
      }
    }
  }
  x1 <- merge(IAT.df, neutral.df)
  all.IAT <<- merge(x1, canonical.df)
  Plot_pKa2Vdss_Effects(all.IAT, outfile = "pKaVdss_minimal.tiff", x.coord.caption = "Variable IAT")

  return(IAT.df)
}

##########
## Plot the boxplot of values + neutral + canonical.
##########
Plot_pKa2Vdss_Effects <- function(IAT.df, outfile = "plot.png", x.coord.caption = "Chemical", y.coord.caption = "Volume of Distribution")
{
  save.height <- 5*length(unique(IAT.df$Name)) / 15   ## Not sure if the InferIAT works with this. If not, save.height=5
  VDSS.DF <<- IAT.df
  p <- ggplot() + geom_boxplot(data=IAT.df, aes(x=Name, y=mod)) 
  p <- p + geom_point(data=IAT.df, aes(x=Name, y=neutral), colour="blue", size=3) 
  p <- p + theme(axis.text.x=element_text(angle=90, vjust=0.5)) 
  p <- p + geom_point(data=IAT.df, aes(x=Name, y=canonical), size=3, colour="red")
  p <- p + coord_flip() + scale_x_discrete(name=x.coord.caption) 
  p <- p + scale_y_continuous(name=y.coord.caption)
  VDSS <<- p
  plot(p, main=paste0("Effect of varying pKa on Volume of Distribution"), ylab="Volume of Distribution", xlab="")
  ggsave(filename=outfile, dpi=450, width=7.5, height=save.height, unit="in")

  ## 122015: Load the df's from the agnostic and minimal analyses. cbind their type "agnostic" and "minimal" to these
  ## df's, and then pasted the two together into all.analyses, where the facet_grid separates the two classes to make the
  ## coolest figure in the universe.
  ##
  ## For the updated figure:
  ## Loaded both the minimum and agnostic df
  #load("pKaVdss_agnostic_data.RData")
  #agnostic.df <- VDSS.DF
  #load("pKaVdss_minimal_data.RData")
  #analysis_type <- rep("agnostic", nrow(agnostic.df))
  #agnostic.df <- cbind(agnositic.df, agnostic_analysis)
  #minimal_analysis <- rep("minimal", nrow(VDSS.DF))
  #minimal.df <- cbind(VDSS.DF, minimal_analysis)
  #minimal.df <- minimal.df[,!(names(minimal.df) == "StringsAsFactors")]
  #all.analyses <- rbind(agnostic.df, minimal.df)
  #p <- ggplot() + geom_boxplot(data=all.analyses, aes(x=Name, y=mod)) 
  #p <- p + geom_point(data=all.analyses, aes(x=Name, y=neutral), colour="blue", size=3) 
  #p <- p + theme(axis.text.x=element_text(angle=90, vjust=0.5, size=10), axis.text.y=element_text(size=7)) 
  #p <- p + geom_point(data=all.analyses, aes(x=Name, y=canonical), size=3, colour="red")
  #p <- p + coord_flip() + scale_x_discrete(name="Chemical / Variable IAT") 
  #p <- p + scale_y_log10(name="Apparent Volume of Distribution", breaks=c(1, 10, 50, 100)) + facet_grid(. ~ analysis_type)
  #ggsave(filename="pKaVdss_both.tiff", dpi=300, width=7.25, height=5, unit="in")
}

##########
## For a given chemical set (the mod.chem with pKa's set different), calculated the Vdss for all chems and return
##########
SensitivityAnalysis <- function(phys.o, data.o) {
  dataset.Vdss <- list(neutral = numeric(), canonical = numeric(), mod.chem = rep(0, length(data.o$chemList)))
  for (i in seq_along(data.o$chemList)) {
    ## For each "new" chemical, get all PCs for the tissue
    tissue.PC <- list(tissue = names(phys.o$tissueList))
    for (tissue in seq_along(phys.o$tissueList)) {
      if (phys.o$tissueList[[tissue]]$tissueType != "RBC") {
        newPC <- PartitionCoefficient$new()
        newPC$predPC(phys.o$tissueList[[tissue]], data.o$chemList[[i]])
        tissue.PC[[phys.o$tissueList[[tissue]]$tissueType]] <- c(tissue.PC[[phys.o$tissueList[[tissue]]$tissueType]], newPC)
        #newPC$reportPC(phys.o$tissueList[[tissue]], data.o$chemList[[i]], NULL)
      }
    }
    ## Then calculate the volume of distribution
    indf <- CalculateVolumeofDistribution(phys.o, data.o$chemList[[i]], tissue.PC)
    dataset.Vdss$mod.chem[i] <- indf$Kp
  }
  ## The neutral and canonical forms of the chemical are the first two elements.
  dataset.Vdss$neutral <- dataset.Vdss$mod.chem[1]		## These are the partition coefficient objects, so need to subset the P.tb.
  dataset.Vdss$canonical <- dataset.Vdss$mod.chem[2]
  dataset.Vdss$mod.chem <- dataset.Vdss$mod.chem[-c(1,2)]

  cat("neutral:", dataset.Vdss$neutral, "\n")
  cat("canonical:", dataset.Vdss$canonical, "\n")

  return(dataset.Vdss)
}


#####
## Calculate Vdss of a compound when pKa is variable
#####
pKaSensitivity <- function(chemical, which.pKa.row, pKa.draw.list, reps) 
{
    out.data <- Data$new()
    for (i in 1:reps) {
      ## Add chemical to the list reps times.
      out.data$chemList <- c(out.data$chemList, chemical$copy())
    }
    ## Modify the pKas based on distribution... except for first and second one: Neutral and normal, respectively
    out.data$chemList[[1]]$pKa$pKa_df <- out.data$chemList[[1]]$pKa$pKa_df[-c(1:nrow(out.data$chemList[[1]]$pKa$pKa_df)),]
    ## no pKa above, setting start value at 3 below keeps second one pristine.
    samples <- sample(pKa.draw.list, reps, replace = TRUE)
    for (i in 3:reps) {
      ## For all other pKa, draw new values.
      out.data$chemList[[i]]$pKa$pKa_df$value[which.pKa.row] <- samples[i]
    }
    return(out.data)
}
#####
## A sensitivity analysis for the most difficult case: Inferring pKa positions, IATs, and pKa Values.
## Will calculate for all of the input Vdss chemicals.
#####
Vdss_SensitivityAnalysis_InferIAT <- function(data.o, phys.o, reps=5000, IAT.list=NULL)
{
  LoadIAResources()
  IAT_hists <- c(1:9, 11, 69, 72)
  ## Load the IATs, their relative frequency, and their list of chemical values ##
  if (is.null(IAT.list)) {
    ## No. This is a small df, not the ExpU.
  	flat.df <- data.o$flatten.pKa()
  	IAT.list <<- GetMonteCarloIATInfo(flat.df, IAT_hists)
  }

  n <- length(data.o$chemList) * reps
  IAT.df <- data.frame(Name = rep(NA, n), mod = rep(NA, n), neutral = rep(NA, n), canonical = rep(NA, n), stringsAsFactors=FALSE)

  aggregate_results <- list()
  for (chem in seq_along(data.o$chemList)) {
    cat("Chemical:", chem, "\n")
    #####
    ## Identification
    #####
    NAME <- data.o$chemList[[chem]]$ID$Compound

    #####
    ## Neutral, canonical
    #####
    ## Canonical
    PC <- CalculatePartitioning(phys.o, data.o$chemList[[chem]])
    CANONICAL <- CalculateVolumeofDistribution(phys.o, 
                                               data.o$chemList[[chem]], 
                                               PC
                                              )$Kp
    ## Neutral
    neutral.chem <- data.o$chemList[[chem]]$copy()
    pKa.SMILES <- data.o$chemList[[chem]]$structure$SMILES
    neutral.chem$pKa <- pKa$new(pKa_df = data.frame(), 
                                pKa_SMILES=SMILES$new(SMILES=pKa.SMILES))
    PC <- CalculatePartitioning(phys.o, neutral.chem)
    NEUTRAL <- CalculateVolumeofDistribution(phys.o, neutral.chem, PC)$Kp

    #####
    ## Create a new data frame for the analysis, 1 per chemical (DF will have
    ## 'reps' chemicals)
    #####
    newData <- perturb.pKa(data.o$chemList[[chem]], reps)
    ## Calculate the volume of distribution for the set of data.
    j <- (chem - 1) * reps    # offset in data.frame for chemical.
    for (i in seq_along(newData$chemList)) {
      ## Calculate the partition coefficents, then volume of distribution
      partition.coefficients <- CalculatePartitioning(phys.o, 
                                                      newData$chemList[[i]])
      IAT.df$mod[i+j] <- CalculateVolumeofDistribution(phys.o, 
                                                       newData$chemList[[i]], 
                                                       partition.coefficients
                                                      )$Kp
      IAT.df$neutral[i+j] <- NEUTRAL
      IAT.df$canonical[i+j] <- CANONICAL
      IAT.df$Name[i+j] <- NAME
    }
  }
  Plot_pKa2Vdss_Effects(IAT.df, 
                        outfile = "pKaVdss_agnostic.tiff", 
                        x.coord.caption = "Chemical", 
                        y.coord.caption = "Volume of Distribution")

  return(IAT.df)
}

#####
##
#####
perturb.pKa <- function(chemical, reps) {
  ## New data object; chemList built in this function, each chem with different
  ## pKa (if IAT exists)
  pKa_varied.o <- Data$new()

  ## Make the pKa object for the original chemical. Below, 
  ## determine the potential ionizable sites for the chemical, 
  ## setting value to NA; to be chosen based on IAT distribution in loop.
  pKa.SMILES <- chemical$structure$SMILES
  pKa.orig.o <- pKa$new(pKa_df = data.frame(), 
                        pKa_SMILES=SMILES$new(SMILES=pKa.SMILES))
  pKa.orig.o$pKa_df <- putativeIAT(pKa.SMILES)

  #####
  ## Reaching this point means that there are some pKa. Choose pKa + set from
  ## dist here.
  #####
	for (i in seq(reps)) {
    ## Create new chemical, but has the same properties as the input chemical.
    mod.chem <- chemicalProperties$new(chemical)
    ## Copy
    pKa.o <- pKa.orig.o$copy()
    ## Get the IATs that will be activated based upon the identified putative
    ## IATs found above
    mod.chem$pKa <- choose.IAT(pKa.o)
    pKa_varied.o$chemList <- c(pKa_varied.o$chemList, mod.chem)
  }
  return(pKa_varied.o)
}

##########
## Entry point for calculating the partition coefficients for a physiology/chemical
##########
CalculatePartitioning <- function(physiology, chemical)
{
  tissue.PC <- list(tissue = names(physiology$tissueList))
  for (tissue in seq_along(physiology$tissueList)) {
    tissue.o <- physiology$tissueList[[tissue]]
    if (tissue.o$tissueType != "RBC") {
      newPC <- PartitionCoefficient$new()
      tissue.PC[[tissue.o$tissueType]] <- c(tissue.PC[[tissue.o$tissueType]], 
                                            SetPartitionCoefficient(tissue.o, 
                                                                    chemical)
                                           )
    }
  }
  return(tissue.PC)
}

#####
## Helper function for CalculatePartitioning
## Small function to take a tissue object and chemical object and calculate the 
## partition coefficient for the pair.
#####
SetPartitionCoefficient <- function(tissue, chemical)
{
  newPC <- PartitionCoefficient$new()
  newPC$predPC(tissue, chemical)
  return(newPC)
}

##########
## Helper function to abstract the technicalities of choosing the IATs. In 
## particular, this function is most helpful when a specific atom has multiple
## potential IATs; it will determine which IAT is active for a site, and then
## does a coin toss to determine if it is active. If so, then it gets
## aggregated into a pKa_df to return, otherwise, it is left alone.
##########
choose.IAT <- function(putative_pKa.o) {
  ## pKa object to build data frame, etc, and return to perturb.pKa
  if (nrow(putative_pKa.o$pKa_df) == 0) {
    ## No reason to check for IATs. ##
    return(putative_pKa.o)
  }
  for (j in 1:nrow(putative_pKa.o$pKa_df)) {
    chem.IAT.type <- ChooseIATtype(putative_pKa.o$pKa_df$IAT.type[j], IAT.list)
    chem.IAT.type.id <- which(IAT.list$IAT.type == chem.IAT.type)
    if (will.ionize(IAT.list$percent.ionized[chem.IAT.type.id]) ) {
      sampled.pKa <- sample(IAT.list$pKa.list[[chem.IAT.type.id]], 1)
      putative_pKa.o$pKa_df$value[j] <- sampled.pKa
    } 
  }
  not.is.na_pKa_ <- which(!is.na(putative_pKa.o$pKa_df$value))
  putative_pKa.o$pKa_df <- putative_pKa.o$pKa_df[not.is.na_pKa_,]
  return(putative_pKa.o)
}

#####
## will.ionize:
## This function stochastically determines whether a particular IAT.type will 
## ionize based on the empirical observation of the ionization probability in
## the 32K data set. 
#####
will.ionize <- function(percent)
{
  if (runif(n=1, min=0, max=1) < percent) {
  	return(TRUE)
  }
  return(FALSE)
}

#####
## This function will determine the most ionizable IAT type in a list of IATs,
## and will return that IAT. In other words, it is a battle of IATs; this 
## could also be done by getting the relative ionization potential of each, 
## and determining the ratio of occasions where the ionization will occur, 
## but I am not going to deal with this at the moment.
#####
ChooseIATtype <- function(IAT.vec, IAT.list) 
{
  IATs <- unlist(strsplit(IAT.vec, ";"))
  idx <- which(IAT.list$IAT.type %in% IATs)
  ## All of the listed ionization types should be possible; based on their 
  ## percentage chance of being ionized, choose the IAT type.
  if (length(IATs) > 1) {
    total.percent.ionized <- sum(IAT.list$percent.ionized[idx])
    probabilities <- IAT.list$percent.ionized[idx] / total.percent.ionized
    IAT.sample <- sample(IAT.list$IAT.type[idx], 
                         replace=TRUE, 
                         size = 1, 
                         prob = probabilities)
  } else {
    IAT.sample <- IATs
  }
  return(IAT.sample)
}