interstitial <- setRefClass("interstitial",
  fields = list(
  	Fm = "numeric",
  	Fnlm = "numeric",
    Fnplm = "numeric",
  	Faplm = "numeric",
  	Fwm = "numeric",
  	Falbp = "numeric",
  	Flprp = "numeric",
  	pH = "numeric"
  ),
  methods = list(
  	setIT = function(df) {
  	  cols <- names(df)
  	  for(i in 1:length(cols)) {
  	  	if (toupper(cols[i]) %in% toupper(names(.self$.refClassDef@fieldClasses))) {
  	  	  set_stmt <- paste0('.self$', cols[i], '<- df$', cols[i])
  	  	  eval(parse(text=set_stmt))
  	  	}
  	  }
  	}
  )
)

cellular <- setRefClass("cellular",
  fields = list(
  	Fm = "numeric",
  	Fnlm = "numeric",
  	Fnplm = "numeric",
  	Faplm = "numeric",
  	Fwm = "numeric",
  	Fprm = "numeric",
  	pH = "numeric"
  ),
  methods = list(
  	setCT = function(df) {
  	  cols <- names(df)
  	  for(i in 1:length(cols)) {
  	  	if (toupper(cols[i]) %in% toupper(names(.self$.refClassDef@fieldClasses))) {
  	  	  set_stmt <- paste0('.self$', cols[i], '<- df$', cols[i])
  	  	  eval(parse(text=set_stmt))
  	  	}
  	  }
  	}
  )
)

Tissue <- setRefClass("Tissue",
  fields = list(
 	  tissueType = "character",
    tissueSpecies = "character",
    tissueVolume = "numeric",
    tissueFlow = "numeric",
  	interstitial.fluid = "interstitial",
  	cellular.tissue = "cellular",
    plasma = "interstitial",
    erythrocyte = "cellular",
    F = "list"
  ),
  methods = list(
  	setTissue = function(tissue.type, tissue.phys.df, ct.df, it.df, e.df, p.df) {
      ## Set character/numeric constants
  	  .self$tissueType <- tissue.type
      .self$tissueSpecies <- tissue.phys.df$species
      .self$tissueVolume <- tissue.phys.df$volume
      .self$tissueFlow <- tissue.phys.df$flow
      ## Set objects (in as .df)
  	  .self$interstitial.fluid <- interstitial$new()
  	  .self$cellular.tissue <- cellular$new()
      .self$plasma <- interstitial$new()
      .self$erythrocyte <- cellular$new()
  	  .self$interstitial.fluid$setIT(it.df)
  	  .self$cellular.tissue$setCT(ct.df)
      .self$plasma$setIT(p.df)
      .self$erythrocyte$setCT(e.df)
  	  .self$setMatrices()
  	},
    setMatrices = function() {
      .self$F <- list(# Interstitium
        it=interstitial.fluid$Fm,                                                                       ## Fraction
        nlit=interstitial.fluid$Fnlm, nplit=interstitial.fluid$Fnplm, aplit=interstitial.fluid$Faplm,   ## Lipids
        wit=interstitial.fluid$Fwm,                                                                     ## Water
        albit=interstitial.fluid$Falbp, lprit=interstitial.fluid$Flprp,                                 ## Protein
        pHit=interstitial.fluid$pH,                                                                     ## Ionization potential.
        ## Cellular
        ct=cellular.tissue$Fm, 
        nlct=cellular.tissue$Fnlm, nplct=cellular.tissue$Fnplm, aplct=cellular.tissue$Faplm, 
        wct=cellular.tissue$Fwm, 
        prct=cellular.tissue$Fprm, 
        pHct=cellular.tissue$pH,
        ## Plasma
        p=plasma$Fm, 
        nlp=plasma$Fnlm, nplp=plasma$Fnplm, aplp=plasma$Faplm, 
        wp=plasma$Fwm, 
        albp=plasma$Falbp, lprp=plasma$Flprp, 
        pHp=plasma$pH,
        ## RBC
        e=erythrocyte$Fm, 
        nle=erythrocyte$Fnlm, nple=erythrocyte$Fnplm, aple=erythrocyte$Faplm, 
        we=erythrocyte$Fwm, 
        pre=erythrocyte$Fprm, 
        pHe=erythrocyte$pH
      )
    }
  )
)

