## Routine that goes from predictor_pKa_by_chemical.flat all the way to plots ##
library(ggplot2)
#attach("Projects//pK-2014-09-15/pK/projects//pKa-format//data//RData//MARVIN_df.RDATA")

plotIonizablePerAtom <- function(predictor_pKa_by_chemical.flat, by.chemclass = FALSE) {
  .e = environment()    ## Otherwise the plot command gets really quite confused when looking for ANY variable (variable not found)

  # Separate into chem-specific
  if (by.chemclass) {
    predictor_pKa_by_chemical.Rx <- predictor_pKa_by_chemical.flat[which(predictor_pKa_by_chemical.flat$chem_class == "Rx"),]
    predictor_pKa_by_chemical.near <- predictor_pKa_by_chemical.flat[which(predictor_pKa_by_chemical.flat$chem_class == "env.near"),]
    predictor_pKa_by_chemical.far <- predictor_pKa_by_chemical.flat[which(predictor_pKa_by_chemical.flat$chem_class == "env.far"),]
  }

  first_atom_type <- which(names(predictor_pKa_by_chemical.flat) == "AlHdrxl_.OH")
  last_atom_type <- which(names(predictor_pKa_by_chemical.flat) == "Barbitur_C1C..O.NC..O.NC1.O")

  # Means for each column (could also do colSums, if it would be useful later) 
  ## If chemical classes are unnecessary, then get the colMeans of the entire flat data frame.
  ## colMeans will return the average value for each of the ionizable atom types. These will produce the points on the 
  ## dotplot that determines the pKa's of interest in our analyses. (cutoff ~.10 per atom)
  if (by.chemclass) {
    Rx.chemcomp <- colMeans(predictor_pKa_by_chemical.Rx[,first_atom_type:last_atom_type], na.rm=TRUE)
    near.chemcomp <- colMeans(predictor_pKa_by_chemical.near[,first_atom_type:last_atom_type], na.rm=TRUE)
    far.chemcomp <- colMeans(predictor_pKa_by_chemical.far[,first_atom_type:last_atom_type], na.rm=TRUE)
    tchemComp <- data.frame(Rx=Rx.chemcomp, near=near.chemcomp, far=far.chemcomp, stringsAsFactors=FALSE)
  } else {
    all.chemcomp <- colMeans(predictor_pKa_by_chemical.flat[, first_atom_type:last_atom_type], na.rm=TRUE)
    tchemComp <- data.frame(all=all.chemcomp, stringsAsFactors = FALSE)
  }


  ## Global plot variable.
  immigration_theme <- theme_update(axis.text.x=element_text(angle=90, hjust=1))
  ## All ionized atoms plot: 
  if (by.chemclass) {
    colorder <- tchemComp[order(tchemComp$Rx, tchemComp$near, tchemComp$far),]
    #a <- ggplot(colorder, aes(x=1:nrow(colorder)), environment = .e) + geom_point(aes(y=Rx, colour="Rx")) 
    #a <- a + geom_point(aes(y=near, colour="Near-field")) + geom_point(aes(y=far, colour="Far-field"))
    #a <- a + scale_x_discrete(limits=1:nrow(colorder), labels=rownames(colorder), name="Ionizable atom type")
    #a <- a + scale_y_continuous(name="Average number per chemical")
    #a <- a + theme(legend.title=element_blank(), axis.text.x = element_text(size=8), axis.text.y=element_text(size=12), axis.title.y=element_text(size=14), axis.title.x=element_text(size=14))
    #png("Ionizable_atom_type_by_chem_class-full.png")
    #plot(a, xlab="Ionizable atom type", ylab="Average occurrences/compound")
    #dev.off()
    #ggsave(filename="Ionizable_atom_type_by_chem_class-full.png", plot=a)

    ## Subset of ionized atoms that are somewhat interesting
    small_colorder <- colorder[56:nrow(colorder),]
    b <- ggplot(small_colorder, aes(x=1:nrow(small_colorder)), environment = .e) + geom_point(aes(y=Rx, colour="Rx"))
    b <- b + geom_point(aes(y=near, colour="Near-field")) + geom_point(aes(y=far, colour="Far-field"))
    b <- b + scale_x_discrete(limits=1:nrow(small_colorder), labels=rownames(small_colorder), name="Ionizable atom type")
    b <- b + scale_y_continuous(name="Average number per chemical")
    b <- b + theme(legend.title=element_blank(), axis.text.x = element_text(size=10), axis.text.y=element_text(size=12), axis.title.y=element_text(size=14), axis.title.x=element_text(size=14))
    plot(b, xlab="Ionizable atom type", ylab="Average occurrences/compound")
    ggsave("Ionizable_atom_type_by_chem_class-small.tiff", dpi=450, width=6, height=6, unit="in")
  } else {
    ordered.vals <- order(tchemComp$all)
    colorder <- as.data.frame(tchemComp[ordered.vals,])
    rownames(colorder) <- rownames(tchemComp)[ordered.vals]

    small_colorder <- as.data.frame(colorder[56:nrow(colorder),])
    colnames(small_colorder) <- "all"
    rownames(small_colorder) <- rownames(colorder)[56:nrow(colorder),]
    b <- ggplot(small_colorder, aes(x=1:nrow(small_colorder)), environment = .e) + geom_point(aes(y=all))
    b <- b + scale_x_discrete(limits=1:nrow(small_colorder), labels=rownames(small_colorder), name="Ionizable atom type")
    b <- b + scale_y_continuous(name="Average number per chemical")
    b <- b + theme(legend.title=element_blank(), axis.text.x = element_text(size=10), axis.text.y=element_text(size=12), axis.title.y=element_text(size=14), axis.title.x=element_text(size=14))
    plot(b, xlab="Ionizable atom type", ylab="Average occurrences/compound")
    ggsave("percent_ionizable_atom_type_per_chemical.tiff", dpi=450, width=6, height=6, unit="in")
  }
}

#####
## Plots the distribution of predictions for a given predictor with respect to the chemical classes.
#####
plotPredictorByChemClass <- function(predictor_pKa_by_chemical.flat) {
  ## Histograms for pKas by method
  c <- hist(as.numeric(predictor_pKa_by_chemical.flat$pKa[which(!is.na(predictor_pKa_by_chemical.flat$pKa))]), breaks=300, xlim=range(-5,18))
#  c <- c + scale_x_discrete(limits=-5:15, name="pH")
  plot(c, xlab="pH", ylab="Frequency of pKa prediction", main="", xlim=range(-5,18))
  c.Rx <- hist(as.numeric(predictor_pKa_by_chemical.flat$pKa[which(!is.na(predictor_pKa_by_chemical.flat$pKa) & predictor_pKa_by_chemical.flat$chem_class == "Rx")]), breaks=300, xlim=range(-5,18))
  png("predictor_pKa_by_Rx.png")
  plot(c.Rx, xlab="pH", ylab="Frequency of pKa prediction", main="Rx", xlim=range(-5,18))
  dev.off()
  c.near <- hist(as.numeric(predictor_pKa_by_chemical.flat$pKa[which(!is.na(predictor_pKa_by_chemical.flat$pKa) & predictor_pKa_by_chemical.flat$chem_class == "env.near")]), breaks=300, xlim=range(-5,18))
  png("predictor_pKa_by_nearField.png")
  plot(c.near, xlab="pH", ylab="Frequency of pKa prediction", main="Near-field", xlim=range(-5,18))
  dev.off()
  c.far <- hist(as.numeric(predictor_pKa_by_chemical.flat$pKa[which(!is.na(predictor_pKa_by_chemical.flat$pKa) & predictor_pKa_by_chemical.flat$chem_class == "env.far")]), breaks=300, xlim=range(-5,18))
  png("predictor_pKa_by_farField.png")
  plot(c.far, xlab="pH", ylab="Frequency of pKa prediction", main="Far-field", xlim=range(-5,18))
  dev.off()
}

#####
## This function is used to compare the pKa Plugin, ADMET Predictor, and SPARC distributions of ALL predicted pKa
## This will be the only comparison between the methods in the manuscript, since the pKa type is unnecessary to get distribution.
#####
plotPKADistByPredictor <- function(reread.data = FALSE)    ## Generally we would like to skip reading data, unless we know that we need to.
{
  reflatten.MARVIN <- reflatten.SPARC <- reflatten.ADMETPred <- FALSE

  if (reread.data) {
    MARVIN.o <<- PBPKxIonSetChemical()
    SPARC.o <<- PBPKxIonSetChemical(pKa.data.sheet = "SPARC")
    ADMETPred.o <<- PBPKxIonSetChemical(pKa.data.sheet = "ADMETPred")
    reflatten.MARVIN <- reflatten.SPARC <- reflatten.ADMETPred <- TRUE
  }

  ## All of these are time consuming, so I would prefer not to do them if I don't have to. ##
  if (!exists("MARVIN.o")) {
    MARVIN.o <<- PBPKxIonSetChemical()
    reflatten.MARVIN <- TRUE
  }
  if (!exists("SPARC.o")) {
    SPARC.o <<- PBPKxIonSetChemical(pKa.data.sheet = "SPARC")
    reflatten.SPARC <- TRUE
  }
  if (!exists("ADMETPred.o")) {
    ADMETPred.o <<- PBPKxIonSetChemical(pKa.data.sheet = "ADMETPred")
    reflatten.ADMETPred <- TRUE
  }

  if (!exists("MARVIN.flat") | reflatten.MARVIN) {
    MARVIN.flat <<- MARVIN.o$flatten.pKa()
  }
  if (!exists("SPARC.flat") | reflatten.SPARC) {
    SPARC.flat <<- SPARC.o$flatten.pKa()
  }
  if (!exists("ADMETPred.flat") | reflatten.ADMETPred) {
    ADMETPred.flat <<- ADMETPred.o$flatten.pKa()
  }

  keeps <- c("CASRN", "pKa", "is.near", "is.far", "is.Rx")

  MARVIN.all <- MARVIN.flat[,keeps]
  SPARC.all <- SPARC.flat[,keeps]
  ADMETPred.all <- ADMETPred.flat[,keeps]

  ## Need to span the entire range of predictions for all prediction methods. This will do, since 
  ## xlim is being set to the appropriate range of interest.

  MARVIN.all$predictor <- "pKa Plugin"
  SPARC.all$predictor <- "SPARC"
  ADMETPred.all$predictor <- "ADMET Predictor"

  all <- rbind(MARVIN.all, SPARC.all)
  all <- rbind(all, ADMETPred.all)
  all$group <- "Total"

  class <- all
  class$group[which(class$is.Rx == TRUE)] <- "Rx"
  class$group[which(class$is.Rx == FALSE & class$is.near == TRUE)] <- "Near-field"
  class$group[which(class$is.Rx == FALSE & class$is.near == FALSE)] <- "Far-field"

  final <- rbind(all, class)

  p <- ggplot(final, aes(x=pKa)) + geom_histogram(binwidth=0.25) + scale_x_continuous(limits=c(-10, 20), name="pH") + 
       facet_grid(group~predictor, scales="free")  ## space="free_y"

  plot(p)
  ggsave("compare_pKa_dist.tiff", dpi=450, width=5.90735, height=6.09265, units="in")

  MARVIN.avb <- MARVIN.flat[, c("pKa", "class", "chem_class")]
  MARVIN.avb$class[which(MARVIN.avb$class == 2)] <- "basic"
  MARVIN.avb$class[which(MARVIN.avb$class == 1)] <- "acidic"
  MARVIN.avb$chem_class[which(MARVIN.avb$chem_class == "env.far")] <- "Far-field"
  MARVIN.avb$chem_class[which(MARVIN.avb$chem_class == "env.near")] <- "Near-field"
  
  p <- ggplot(MARVIN.avb, aes(x=pKa, fill=class)) + geom_histogram(binwidth = 0.25, position = "dodge")
  p <- p + scale_x_continuous(limits = c(-10, 20), name = "pH")
  p <- p + facet_grid(chem_class ~ ., scales = "free")
  plot(p)
  ### 3.65: a little less than half max width (7.5).
  ### 4.3: a little less than half max height (8.75).
  ggsave("acid_base_distribution.tiff", dpi=450, width=6, height=4.3, units="in")

}

#####
## Plot the distribution of acids vs. bases for each of the chemical classes
#####
histogram.acid.base.distribution <- function(df, chem_class = "Rx") {
  if(chem_class == "Rx") {
    hist_data <- df[which(df$is.Rx),]
    title = "Rx"
  } else if (chem_class == "near") {
    hist_data <- df[which(df$is.near & !df$is.Rx),]
    title="Near-field"
  } else {
    hist_data <- df[which(df$is.far & !df$is.near & !df$is.Rx),]
    title="Far-field"
  }

  hist_data$type <- sub("1", "acid", hist_data$type)
  hist_data$type <- sub("2", "base", hist_data$type)

  a <- ggplot(data=hist_data, aes(x=pKa, fill=type)) 
  a <- a + geom_histogram(binwidth=0.25, position="dodge") 
  a <- a + scale_x_continuous(limits=c(0,15), name="pH") 
  a <- a + scale_y_continuous(name="# pKa of each type") 
  a <- a + ggtitle(title) 
  a <- a + theme(legend.title=element_blank(), plot.title = element_text(face="bold", size=32), axis.text.x = element_text(size=20), axis.text.y=element_text(size=20), axis.title.y=element_text(size=24), axis.title.x=element_text(size=24))
  ggsave(filename=paste0("barplot_", chem_class, ".png"), plot=a)
}

#####
## Plot all
## For manuscript: c(1:9, 11, 69, 72)
## 12162015: To use this function, df is the result of pKaTools.df$flatten.pKa
## Plots the IAT distributions for the histograms of interest. Density plots are the basis of
## the figure of manuscript plots.
#####
plot.IAT.Distribution <- function(df, needed_hists, plot = TRUE, chem_classes = FALSE) {

  hist_data <- CollectIATDistributions4Plot(df, needed_hists)
  test_hist <<- hist_data

  ## I have no idea why, but this plots only 1 graph in code, but when copied and pasted into RStudio, 
  ## plots it all. ??
  #if (chem_classes) {
  #  a.hist <- ggplot(hist_data, aes(x=pKa, fill=chem_class)) 
  #} else {
  #  a.hist <- ggplot(hist_data, aes(x=pKa))
  #}
  #a.hist <- a.hist + geom_histogram(binwidth=0.5, position="dodge") 
  #a.hist <- a.hist + scale_x_continuous(limits=c(-10,20), name="pH") 
  #a.hist <<- a.hist + facet_wrap(~IAT, ncol=3)
  #plot(a.hist)
  #ggsave(filename=paste0("IAT.tiff"), dpi = 300, plot=a.hist, width=4, height=6, units = "in")
  ## DENSITY PLOT VERSION:
  if (chem_classes) {
    a.dens <- ggplot(test_hist, aes(x=pKa, fill=chem_class)) 
    a.dens <- a.dens + geom_density(aes(color=chem_class), alpha=0.1) 
  } else {
    a.dens <- ggplot(test_hist, aes(x=pKa))    
    a.dens <- a.dens + geom_density(alpha=0.1) 
  }
  a.dens <- a.dens + scale_x_continuous(limits=c(-10,20), name="pH") 
  a.dens <<- a.dens + facet_wrap(~IAT, ncol=3) #, scales="free")
  plot(a.dens)
  ggsave(filename=paste0("IAT_dens.tiff"), dpi = 300, plot=a.dens, width=4, height=6, units = "in")
}

CollectIATDistributions4Plot <- function(df, needed_hists)
{
  LoadIAResources()
  first_atom_type <- which(names(df) == "AlHdrxl_.OH")
  last_atom_type <- which(names(df) == "Barbitur_C1C..O.NC..O.NC1.O")
  ionizable_atom.df <- df[,first_atom_type:last_atom_type]
  for (i in 1:length(needed_hists)) {
    hist_collect <- count.ionizable.atom.types(df, needed_hists[i])
    hist_collect$IAT <- ia_thesaurus[[needed_hists[i]]]
    if (i == 1) { hist_data <- hist_collect }
    else { hist_data <- rbind(hist_data, hist_collect) }
  }

  return(hist_data)
}

CollectIATDistributions <- function(df, needed_hists)
{
  LoadIAResources()
  ret.df <- data.frame(IAT.type=NULL, pKa.dist=NULL)
  first_atom_type <- which(names(df) == "AlHdrxl_.OH")
  last_atom_type <- which(names(df) == "Barbitur_C1C..O.NC..O.NC1.O")
  ionizable_atom.df <- df[,first_atom_type:last_atom_type]
  for (i in 1:length(needed_hists)) {
    hist_data <- count.ionizable.atom.types(df, needed_hists[i])
    ## For GetMonteCarloIAT. This is stupid. ##
    if (nrow(hist_data) > 0) {
      x.df <- data.frame(IAT.type=names(ia_dictionary)[needed_hists[i]], pKa.dist=hist_data$pKa)
      ret.df <- rbind(ret.df, x.df)
    }
  }
  return(ret.df)
}