### Sensitivity analysis.

##########
## Plot the boxplot of values + neutral + canonical.
##########
Plot_pKa2Vdss_Effects <- function(IAT.df, outfile = "plot.png", x.coord.caption = "Chemical", y.coord.caption = "Volume of Distribution")
{
  save.height <- 5*length(unique(IAT.df$Name)) / 15   ## Not sure if the InferIAT works with this. If not, save.height=5
  VDSS.DF <<- IAT.df
  p <- ggplot() + geom_boxplot(data=IAT.df, aes(x=Name, y=mod)) 
  p <- p + geom_point(data=IAT.df, aes(x=Name, y=neutral), colour="blue", size=3) 
  p <- p + theme(axis.text.x=element_text(angle=90, vjust=0.5)) 
  p <- p + geom_point(data=IAT.df, aes(x=Name, y=canonical), size=3, colour="red")
  p <- p + coord_flip() + scale_x_discrete(name=x.coord.caption) 
  p <- p + scale_y_continuous(name=y.coord.caption)
  VDSS <<- p
  plot(p, main=paste0("Effect of varying pKa on Volume of Distribution"), ylab="Volume of Distribution", xlab="")
  ggsave(filename=outfile, dpi=450, width=7.5, height=save.height, unit="in")

  ## 122015: Load the df's from the agnostic and minimal analyses. cbind their type "agnostic" and "minimal" to these
  ## df's, and then pasted the two together into all.analyses, where the facet_grid separates the two classes to make the
  ## coolest figure in the universe.
  ##
  ## For the updated figure:
  ## Loaded both the minimum and agnostic df
  #load("pKaVdss_agnostic_data.RData")
  #agnostic.df <- VDSS.DF
  #load("pKaVdss_minimal_data.RData")
  #analysis_type <- rep("agnostic", nrow(agnostic.df))
  #agnostic.df <- cbind(agnositic.df, agnostic_analysis)
  #minimal_analysis <- rep("minimal", nrow(VDSS.DF))
  #minimal.df <- cbind(VDSS.DF, minimal_analysis)
  #minimal.df <- minimal.df[,!(names(minimal.df) == "StringsAsFactors")]
  #all.analyses <- rbind(agnostic.df, minimal.df)
  #p <- ggplot() + geom_boxplot(data=all.analyses, aes(x=Name, y=mod)) 
  #p <- p + geom_point(data=all.analyses, aes(x=Name, y=neutral), colour="blue", size=3) 
  #p <- p + theme(axis.text.x=element_text(angle=90, vjust=0.5, size=10), axis.text.y=element_text(size=7)) 
  #p <- p + geom_point(data=all.analyses, aes(x=Name, y=canonical), size=3, colour="red")
  #p <- p + coord_flip() + scale_x_discrete(name="Chemical / Variable IAT") 
  #p <- p + scale_y_log10(name="Apparent Volume of Distribution", breaks=c(1, 10, 50, 100)) + facet_grid(. ~ analysis_type)
  #ggsave(filename="pKaVdss_both.tiff", dpi=300, width=7.25, height=5, unit="in")
}

Bootstrap_IAT.Vdss_ <- function(chem.o, 
                                phys.o, 
                                reps, 
                                IAT.profile)
{
  # Data structure for holding the pKa and associated Vdss
  # pKa: all pKa values, separated by ';'
  # Vdss: calculated Vdss value
  # Create a Data object with pseudochemicals.
  # Each pseudochemical has is the same as input chem, but with varied pKa 
  pseudo.chemical <- sample.pKa(chem.o, reps)
  # Create a list for the data.frames created in the loop
  df.list <- list()
  # Calculate the volume of distribution for the set of data.
  for (i in seq_along(pseudo.chemical$chemList)) {
    this.pKa <- pseudo.chemical$chemList[[i]]$pKa$get.pKa()
    ## Calculate the partition coefficents, then volume of distribution
    partition.coefficients <- CalculatePartitioning(
                                phys.o, 
                                pseudo.chemical$chemList[[i]]
                              )
    this.Vdss <- CalculateVolumeofDistribution(
                    phys.o, 
                    pseudo.chemical$chemList[[i]], 
                    partition.coefficients
                 )$Kp
    df <- data.frame(pKa=this.pKa, Vdss=this.Vdss, stringsAsFactors=FALSE)
    df.list[[i]] <- df
  }
  bootstrap.df <- data.table::rbindlist(df.list)  

  return(bootstrap.df)
}

Vdss_SensitivityAnalysis_InferIAT <- function(data.o, phys.o, reps=5000, IAT.list=NULL)
  #####
  ## A sensitivity analysis for the most difficult case: Inferring pKa positions, IATs, and pKa Values.
  ## Will calculate for all of the input Vdss chemicals.
  #####
{
  LoadIAResources()
  IAT_hists <- c(1:9, 11, 69, 72)
  ## Load the IATs, their relative frequency, and their list of chemical values ##
  if (is.null(IAT.list)) {
    ## No. This is a small df, not the ExpU.
  	flat.df <- data.o$flatten.pKa()
  	IAT.list <<- GetMonteCarloIATInfo(flat.df, IAT_hists)
  }

  n <- length(data.o$chemList) * reps
  IAT.df <- data.frame(Name = rep(NA, n), mod = rep(NA, n), 
                       neutral = rep(NA, n), canonical = rep(NA, n),
                       stringsAsFactors=FALSE)

  for (chem in seq_along(data.o$chemList)) {
    cat("Chemical:", chem, "\n")
    #####
    ## Identification
    #####
    NAME <- data.o$chemList[[chem]]$ID$Compound

    #####
    ## Neutral, canonical
    #####
    ## Canonical
    PC <- CalculatePartitioning(phys.o, data.o$chemList[[chem]])
    CANONICAL <- CalculateVolumeofDistribution(phys.o, 
                                               data.o$chemList[[chem]], 
                                               PC
                                              )$Kp
    ## Neutral
    neutral.chem <- data.o$chemList[[chem]]$copy()
    pKa.SMILES <- data.o$chemList[[chem]]$structure$SMILES
    neutral.chem$pKa <- pKa$new(pKa_df = data.frame(), 
                                pKa_SMILES=SMILES$new(SMILES=pKa.SMILES))
    PC <- CalculatePartitioning(phys.o, neutral.chem)
    NEUTRAL <- CalculateVolumeofDistribution(phys.o, neutral.chem, PC)$Kp

    #####
    ## Create a new data frame for the analysis, 1 per chemical (DF will have
    ## 'reps' chemicals)
    #####
    newData <- sample.pKa(data.o$chemList[[chem]], reps)
    ## Calculate the volume of distribution for the set of data.
    j <- (chem - 1) * reps    # offset in data.frame for chemical.
    for (i in seq_along(newData$chemList)) {
      ## Calculate the partition coefficents, then volume of distribution
      partition.coefficients <- CalculatePartitioning(phys.o, 
                                                      newData$chemList[[i]])
      IAT.df$mod[i+j] <- CalculateVolumeofDistribution(phys.o, 
                                                       newData$chemList[[i]], 
                                                       partition.coefficients
                                                      )$Kp
      IAT.df$neutral[i+j] <- NEUTRAL
      IAT.df$canonical[i+j] <- CANONICAL
      IAT.df$Name[i+j] <- NAME
    }
  }
  Plot_pKa2Vdss_Effects(IAT.df, 
                        outfile = "pKaVdss_agnostic.tiff", 
                        x.coord.caption = "Chemical", 
                        y.coord.caption = "Volume of Distribution")

  return(IAT.df)
}

sample.pKa <- function(chemical, reps) {
  ## New data object; chemList built in this function, each chem with different
  ## pKa (if IAT exists)
  pKa_varied.o <- Data$new()

  ## Make the pKa object for the original chemical. Below, 
  ## determine the potential ionizable sites for the chemical, 
  ## setting value to NA; to be chosen based on IAT distribution in loop.
  pKa.SMILES <- chemical$structure$SMILES
  pKa.orig.o <- pKa$new(pKa_df = data.frame(), 
                        pKa_SMILES=SMILES$new(SMILES=pKa.SMILES))
  pKa.orig.o$pKa_df <- putativeIAT(pKa.SMILES)

  #####
  ## Reaching this point means that there are some pKa. Choose pKa + set from
  ## dist here.
  #####
	for (i in seq(reps)) {
    ## Create new chemical, but has the same properties as the input chemical.
    mod.chem <- chemicalProperties$new(chemical)
    ## Copy
    pKa.o <- pKa.orig.o$copy()
    ## Get the IATs that will be activated based upon the identified putative
    ## IATs found above
    mod.chem$pKa <- choose.IAT(pKa.o)
    pKa_varied.o$chemList <- c(pKa_varied.o$chemList, mod.chem)
  }
  return(pKa_varied.o)
}

CalculatePartitioning <- function(physiology, chemical)
# Entry point for calculating the partition coefficients for a 
# physiology/chemical
{
  tissue.PC <- list(tissue = names(physiology$tissueList))
  for (tissue in seq_along(physiology$tissueList)) {
    tissue.o <- physiology$tissueList[[tissue]]
    if (tissue.o$tissueType != "RBC") {
      newPC <- PartitionCoefficient$new()
      tissue.PC[[tissue.o$tissueType]] <- c(tissue.PC[[tissue.o$tissueType]], 
                                            SetPartitionCoefficient(tissue.o, 
                                                                    chemical)
                                           )
    }
  }
  return(tissue.PC)
}

#####
## Helper function for CalculatePartitioning
## Small function to take a tissue object and chemical object and calculate the
## partition coefficient for the pair.
#####
SetPartitionCoefficient <- function(tissue, chemical)
{
  newPC <- PartitionCoefficient$new()
  newPC$predPC(tissue, chemical)
  return(newPC)
}

##########
## Helper function to abstract the technicalities of choosing the IATs. In 
## particular, this function is most helpful when a specific atom has multiple
## potential IATs; it will determine which IAT is active for a site, and then
## does a coin toss to determine if it is active. If so, then it gets
## aggregated into a pKa_df to return, otherwise, it is left alone.
##########
choose.IAT <- function(putative_pKa.o) {
  ## pKa object to build data frame, etc, and return to sample.pKa
  if (nrow(putative_pKa.o$pKa_df) == 0) {
    ## No reason to check for IATs. ##
    return(putative_pKa.o)
  }
  for (j in 1:nrow(putative_pKa.o$pKa_df)) {
    chem.IAT.type <- ChooseIATtype(putative_pKa.o$pKa_df$IAT.type[j], IAT.list)
    chem.IAT.type.id <- which(IAT.list$IAT.type == chem.IAT.type)
    if (will.ionize(IAT.list$percent.ionized[chem.IAT.type.id]) ) {
      sampled.pKa <- sample(IAT.list$pKa.list[[chem.IAT.type.id]], 1)
      putative_pKa.o$pKa_df$value[j] <- sampled.pKa
    } 
  }
  not.is.na_pKa_ <- which(!is.na(putative_pKa.o$pKa_df$value))
  putative_pKa.o$pKa_df <- putative_pKa.o$pKa_df[not.is.na_pKa_,]
  return(putative_pKa.o)
}

#####
## will.ionize:
## This function stochastically determines whether a particular IAT.type will 
## ionize based on the empirical observation of the ionization probability in
## the 32K data set. 
#####
will.ionize <- function(percent)
{
  if (runif(n=1, min=0, max=1) < percent) {
  	return(TRUE)
  }
  return(FALSE)
}

#####
## This function will determine the most ionizable IAT type in a list of IATs,
## and will return that IAT. In other words, it is a battle of IATs; this 
## could also be done by getting the relative ionization potential of each, 
## and determining the ratio of occasions where the ionization will occur, 
## but I am not going to deal with this at the moment.
#####
ChooseIATtype <- function(IAT.vec, IAT.list) 
{
  IATs <- unlist(strsplit(IAT.vec, ";"))
  idx <- which(IAT.list$IAT.type %in% IATs)
  ## All of the listed ionization types should be possible; based on their 
  ## percentage chance of being ionized, choose the IAT type.
  if (length(IATs) > 1) {
    total.percent.ionized <- sum(IAT.list$percent.ionized[idx])
    probabilities <- IAT.list$percent.ionized[idx] / total.percent.ionized
    IAT.sample <- sample(IAT.list$IAT.type[idx], 
                         replace=TRUE, 
                         size = 1, 
                         prob = probabilities)
  } else {
    IAT.sample <- IATs
  }
  return(IAT.sample)
}