from rdkit.Chem import AllChem
from rdkit.Chem import Draw

def ProcessInput(smiles, inPKa):

	# pe = inPKa.split(':')[0]
	# atomNum = int(inPKa.split(':')[3])
	
	# if(pe == 'p'):
	# 	charge = '+'
	# else:
	# 	charge = '-'

	# print ("smiles = %s, Atom Num = %d, Charge = '%s'" % (smiles, atomNum, charge))

	# fSmiles = EditSmiles(smiles, atomNum, charge)

	print ("smiles = %s, CASRN = '%s'" % (smiles, inPKa))

	fSmiles = smiles

	SmilesToInchI(fSmiles)

def EditSmiles(smiles, atomNum, charge):
	
	count = 0
	flag = 0
	finalSmiles = ''

	for i in smiles:

		if(i.isalpha()):
			count += 1

		# print ("count = %d, character = %s, Flag = %d" %(count, i, flag))

		if (count == atomNum and flag == 0):
			finalSmiles += '[' + i + charge + ']'
			flag = 1
		else:
			finalSmiles += i

	# print ( "Smiles before editing is %s" %(smiles))
	print ( "Smiles after editing is %s " % (finalSmiles))

	return finalSmiles

def SmilesToMol(smiles):
	
	mol = AllChem.MolFromSmiles(str(smiles), sanitize = False)

	if mol is None:
		print ("cannot convert smiles to molblock" %(smiles))
	else:
		AllChem.SanitizeMol(mol,sanitizeOps = AllChem.SanitizeFlags.SANITIZE_ALL^AllChem.SanitizeFlags.SANITIZE_ADJUSTHS^AllChem.SanitizeFlags.SANITIZE_FINDRADICALS)
		AllChem.Compute2DCoords(mol)
		s_block = AllChem.MolToMolBlock(mol)

		return s_block

def InchIToSmiles(inchi):
	mol = AllChem.MolFromInchi(inchi, sanitize = False)
	if mol is None:
		print("cannot generate depiction for INCHI = %s " %(inchi) )
	else:
		AllChem.SanitizeMol(mol,sanitizeOps = AllChem.SanitizeFlags.SANITIZE_ALL^AllChem.SanitizeFlags.SANITIZE_ADJUSTHS^AllChem.SanitizeFlags.SANITIZE_FINDRADICALS)
		i_smiles = AllChem.MolToSmiles(mol);
		
		return i_smiles
	

def SmilesToInchI(smiles):
	# print("converting smiles to InchI")
	
	mol = AllChem.MolFromSmiles(str(smiles), sanitize = False)

	if mol is None:
		print ("cannot convert smiles to inchi" %(smiles))
	else:
		AllChem.SanitizeMol(mol,sanitizeOps = AllChem.SanitizeFlags.SANITIZE_ALL^AllChem.SanitizeFlags.SANITIZE_ADJUSTHS^AllChem.SanitizeFlags.SANITIZE_FINDRADICALS)
		AllChem.Compute2DCoords(mol)
		s_inchi = AllChem.MolToInchi(mol, options = '/FixedH')

		# mol2 = AllChem.MolFromSmiles(str(smiles))
		# s_inchi_noopt = AllChem.MolToInchi(mol2)

		# print s_inchi
		# print s_inchi_noopt

		s_inchi_key = AllChem.InchiToInchiKey(s_inchi)
		s_smiles = AllChem.MolToSmiles(mol)

		# print (" Inchi = %s \n InchiKey = %s " % (s_inchi, s_inchi_key))
		# print("converted RDKIT cannonical smiles = %s" %(s_smiles))

		inchi_add = "/i"

		for atom in mol.GetAtoms():	
			charge = atom.GetFormalCharge()
			if charge is not 0:
				# print atom.GetAtomicNum() 
				# print str(atom.GetIdx()) + '\t ' + atom.GetSymbol() + '\t ' + str(atom.GetFormalCharge())
				# print atom.GetTotalValence()
				# print atom.GetNeighbors()

				if charge > 0:
					achar = '+'
				else:
					achar = '-'

				inchi_add = inchi_add + atom.GetSymbol() + achar + ','

		# inchi_add = inchi_add[:-1]
		final_inchi = s_inchi + inchi_add[:-1]

		# print 'Final Inchi = ' + final_inchi

		return {'inchi_with_charge' : final_inchi, 'canonical_smiles': s_smiles, 'inchi_no_charge': s_inchi}

		"""mol2 = AllChem.MolFromInchi(s_inchi, sanitize = False)
		AllChem.SanitizeMol(mol2,sanitizeOps = AllChem.SanitizeFlags.SANITIZE_ALL^AllChem.SanitizeFlags.SANITIZE_ADJUSTHS^AllChem.SanitizeFlags.SANITIZE_FINDRADICALS)
		AllChem.Compute2DCoords(mol2)

		for atom2 in mol2.GetAtoms():	
			charge = atom2.GetFormalCharge()
			if charge is not 0:
				# print atom.GetAtomicNum() 
				print str(atom2.GetIdx()) + '\t ' + atom2.GetSymbol() + '\t ' + str(atom2.GetFormalCharge())
				# print atom.GetTotalValence()
				# print atom.GetNeighbors() """

		# s_block = AllChem.MolToMolBlock(mol) 

		# fname = 'ecg.sdf'

		# fout = open(fname, 'w')

		# fout.write(s_block + '\n')

		# fout.close() 

		# smiles_struct(smiles, 'smi_orig.svg')
		# smiles_struct(s_smiles, 'smi_conv.svg')

		# inchi_struct(s_inchi, 'inchi_changed.png')
		# inchi_struct(s_inchi_noopt, 'inchi_no_opt.png')


def smiles_struct(smiles, fname):
	mol = AllChem.MolFromSmiles(smiles, sanitize = False)
	if mol is None:
		print("cannot generate depiction for SMILES = %s " %( smiles) )
	else:
		AllChem.SanitizeMol(mol,sanitizeOps = AllChem.SanitizeFlags.SANITIZE_ALL^AllChem.SanitizeFlags.SANITIZE_ADJUSTHS^AllChem.SanitizeFlags.SANITIZE_FINDRADICALS)
		AllChem.Compute2DCoords(mol)
		Draw.MolToFile(mol, fname)

def inchi_struct(inchi, fname):
	mol = AllChem.MolFromInchi(inchi, sanitize = False)
	if mol is None:
		print("cannot generate depiction for INCHI = %s " %(inchi) )
	else:
		AllChem.SanitizeMol(mol,sanitizeOps = AllChem.SanitizeFlags.SANITIZE_ALL^AllChem.SanitizeFlags.SANITIZE_ADJUSTHS^AllChem.SanitizeFlags.SANITIZE_FINDRADICALS)
		AllChem.Compute2DCoords(mol)
		Draw.MolToFile(mol, fname, size = (600, 600))
	

result = SmilesToInchI('c1(cc(c(N3CC[N+](C)CC3)cc12)F)C(=O)C(C(O)=O)=CN2CC')

print result

f_smiles = InchIToSmiles(result['inchi_no_charge']);

print f_smiles

# ProcessInput('c1(ccc(c(C(C)=O)c1)OCC(O)CNC(C)C)NC(=O)CCC', 'p:9.7:N:15')
# ProcessInput('c1(ccccc1)CC(=O)NC(C3=O)C(S2)N3C(C(=O)O)C2(C)C', 'N:2.7:n:20')
# ProcessInput('c1(ccc(cc1OC)CCN(C)CCCC(-c2ccc(c(c2)OC)OC)(C#N)C(C)C)OC', 'p:8.5:N:9')
# ProcessInput('CCCC(CCC)C(O)=O', 'N:4.6:n:9')

# ProcessInput('c1(ccccc1C(=O)[N-])OCC', '17902-23-7')
# ProcessInput('CCC1(CC)C(=O)NC(=S)[N-]C1=O', '77-32-7')
# ProcessInput('CCC1=CC=C(C=C1)C(=O)C(C)C[NH+]2CCCC2 ', '99323-21-4')
# ProcessInput('c1(cc(c(N3CC[N+](C)CC3)cc12)F)C(=O)C(C(O)=O)=CN2CC', '51940-44-4')
# ProcessInput('c1(c2N3C=C(C([O-])=O)C(-c2cc(c1N4CC[NH+](C)CC4)F)=O)OCC3C', '70458-92-3')



